@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002-2003, Raphael Manfredi
+ * Copyright (c) 2002-2003, 2014 Raphael Manfredi
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -28,48 +28,64 @@
  * Alive status checking ping/pongs.
  *
  * @author Raphael Manfredi
- * @date 2002-2003
+ * @date 2002-2003, 2014
  */
 
 #include "common.h"
 
 #include "alive.h"
-#include "nodes.h"
-#include "guid.h"
+
 #include "gmsg.h"
+#include "guid.h"
 #include "mq.h"
+#include "nodes.h"
 #include "pcache.h"
+
+#include "core/g2/build.h"
+#include "core/g2/node.h"
+
 #include "lib/atoms.h"
-#include "lib/glib-missing.h"
+#include "lib/eslist.h"
 #include "lib/pmsg.h"
 #include "lib/tm.h"
 #include "lib/walloc.h"
+
 #include "if/gnet_property_priv.h"
+
 #include "lib/override.h"		/* Must be the last header included */
 
 #define INFINITY	0xffffffff
 
+enum alive_magic { ALIVE_MAGIC = 0x778b64bb };
+
 /**
  * Structure used to keep track of the alive pings we sent, and stats.
  */
 struct alive {
-	struct gnutella_node *node;
-	GSList *pings;				/**< Pings we sent (struct alive_ping) */
-	int count;					/**< Amount of pings in list */
-	int maxcount;				/**< Maximum amount of pings we remember */
+	enum alive_magic magic;		/**< Magic number */
+	gnutella_node_t *node;		/**< The node to which we are attached */
+	eslist_t pings;				/**< Pings we sent (struct alive_ping) */
+	time_t last_sent;			/**< When we last sent a ping */
+	uint maxcount;				/**< Maximum amount of pings we remember */
 	uint32 min_rt;				/**< Minimum roundtrip time (ms) */
 	uint32 max_rt;				/**< Maximim roundtrip time (ms) */
 	uint32 avg_rt;				/**< Average (EMA) roundtrip time (ms) */
 	uint32 last_rt;				/**< Last roundtrip time (ms) */
 };
 
+static inline void
+alive_check(const struct alive * const a)
+{
+	g_assert(a != NULL);
+	g_assert(ALIVE_MAGIC == a->magic);
+}
+
 struct alive_ping {
 	const struct guid *muid;	/**< The GUID of the message */
 	tm_t sent;					/**< Time at which we sent the message */
+	slink_t ap_link;			/**< Links sent alive pings */
 };
 
-static void alive_trim_upto(struct alive *a, GSList *item);
-
 /**
  * Create an alive_ping, with proper message ID.
  */
@@ -80,19 +96,28 @@ ap_make(struct guid *muid)
 
 	WALLOC(ap);
 
-	ap->muid = atom_guid_get(muid);
+	ap->muid = NULL == muid ? NULL : atom_guid_get(muid);
 	tm_now_exact(&ap->sent);
 
 	return ap;
 }
 
+/**
+ * Clear an alive ping.
+ */
+static void
+ap_clear(struct alive_ping *ap)
+{
+	atom_guid_free_null(&ap->muid);
+}
+
 /**
  * Free an alive_ping.
  */
 static void
 ap_free(struct alive_ping *ap)
 {
-	atom_guid_free(ap->muid);
+	ap_clear(ap);
 	WFREE(ap);
 }
 
@@ -100,15 +125,24 @@ ap_free(struct alive_ping *ap)
  * Create the alive structure.
  * Returned as an opaque pointer.
  */
-void *
-alive_make(struct gnutella_node *n, int max)
+alive_t *
+alive_make(gnutella_node_t *n, int max)
 {
-	struct alive *a;
+	alive_t *a;
 
 	WALLOC0(a);
+	a->magic = ALIVE_MAGIC;
 	a->node = n;
-	a->maxcount = max;
 	a->min_rt = INFINITY;
+	a->last_sent = tm_time();
+	eslist_init(&a->pings, offsetof(struct alive_ping, ap_link));
+
+	/*
+	 * For G2 nodes, we only allow 1 pending ping: the /PI message has no
+	 * MUID and therefore we cannot know which /PI a /PO would acknowledge.
+	 */
+
+	a->maxcount = NODE_TALKS_G2(n) ? 1 : max;
 
 	return a;
 }
@@ -117,46 +151,29 @@ alive_make(struct gnutella_node *n, int max)
  * Dispose the alive structure.
  */
 void
-alive_free(void *obj)
+alive_free(alive_t *a)
 {
-	struct alive *a = obj;
-	GSList *sl;
+	alive_check(a);
 
-	for (sl = a->pings; sl; sl = g_slist_next(sl))
-		ap_free(sl->data);
-
-	gm_slist_free_null(&a->pings);
+	eslist_foreach(&a->pings, (data_fn_t) ap_clear, NULL);
+	eslist_wfree(&a->pings, sizeof(struct alive_ping));
+	eslist_discard(&a->pings);
+	a->magic = 0;
 	WFREE(a);
 }
 
-/**
- * Remove and free specified "alive ping" list entry.
- */
-static void
-alive_remove_link(struct alive *a, GSList *item)
-{
-	g_assert(a->count > 0);
-	g_assert(a->pings != NULL);
-
-	a->pings = g_slist_remove_link(a->pings, item);
-	a->count--;
-	ap_free(item->data);
-	g_slist_free_1(item);
-}
-
 /**
  * Drop alive ping record when message is dropped.
  */
 static void
-alive_ping_drop(struct alive *a, const struct guid *muid)
+alive_ping_drop(alive_t *a, const struct guid *muid)
 {
-	GSList *sl;
+	struct alive_ping *ap;
 
-	for (sl = a->pings; sl; sl = g_slist_next(sl)) {
-		struct alive_ping *ap = sl->data;
-
-		if (guid_eq(ap->muid, muid)) {		/* Found it! */
-			alive_remove_link(a, sl);
+	ESLIST_FOREACH_DATA(&a->pings, ap) {
+		if (NULL == muid || guid_eq(ap->muid, muid)) {		/* Found it! */
+			eslist_remove(&a->pings, ap);
+			ap_free(ap);
 			return;
 		}
 	}
@@ -170,20 +187,18 @@ static bool
 alive_ping_can_send(const pmsg_t *mb, const void *q)
 {
 	const gnutella_node_t *n = mq_node(q);
-	struct alive *a = n->alive_pings;
+	alive_t *a = n->alive_pings;
 	const struct guid *muid = cast_to_guid_ptr_const(pmsg_start(mb));
 
 	g_assert(gnutella_header_get_function(muid) == GTA_MSG_INIT);
-	g_assert(a->count == 0 || a->pings != NULL);
 
 	/*
 	 * We can send the message only if it's the latest in the list,
 	 * i.e. the latest one we enqueued.
 	 */
 
-	if (a->count) {
-		const GSList *l = g_slist_last(a->pings);
-		const struct alive_ping *ap = l->data;
+	if (0 != eslist_count(&a->pings)) {
+		const struct alive_ping *ap = eslist_tail(&a->pings);
 
 		if (guid_eq(ap->muid, muid))		/* It's the latest one */
 			return TRUE;					/* We can send it */
@@ -196,8 +211,8 @@ alive_ping_can_send(const pmsg_t *mb, const void *q)
 	alive_ping_drop(a, muid);
 
 	if (GNET_PROPERTY(alive_debug))
-		g_debug("ALIVE %s dropped old alive ping #%s, %d remain",
-			node_infostr(a->node), guid_hex_str(muid), a->count);
+		g_debug("ALIVE %s dropped old alive ping #%s, %zd remain",
+			node_infostr(a->node), guid_hex_str(muid), eslist_count(&a->pings));
 
 	return FALSE;		/* Don't send message */
 }
@@ -208,22 +223,39 @@ alive_ping_can_send(const pmsg_t *mb, const void *q)
 static void
 alive_pmsg_free(pmsg_t *mb, void *arg)
 {
-	struct gnutella_node *n = arg;
+	gnutella_node_t *n = arg;
 
 	g_assert(pmsg_is_extended(mb));
 
 	if (pmsg_was_sent(mb)) {
 		n->n_ping_sent++;
-		if (GNET_PROPERTY(alive_debug))
-			g_debug("ALIVE sent ping #%s to %s",
-				guid_hex_str(cast_to_guid_ptr_const(pmsg_start(mb))),
-				node_infostr(n));
+
+		if (GNET_PROPERTY(alive_debug)) {
+			if (NODE_TALKS_G2(n)) {
+				g_debug("ALIVE sent ping to %s", node_infostr(n));
+			} else {
+				g_debug("ALIVE sent ping #%s to %s",
+					guid_hex_str(cast_to_guid_ptr_const(pmsg_start(mb))),
+					node_infostr(n));
+			}
+		}
 	} else {
-		struct guid *muid = cast_to_guid_ptr(pmsg_start(mb));
-		struct alive *a = n->alive_pings;
+		struct guid *muid;
+		alive_t *a = n->alive_pings;
 
 		g_assert(a->node == n);
 
+		muid = NODE_TALKS_G2(n) ? NULL : cast_to_guid_ptr(pmsg_start(mb));
+
+		if (GNET_PROPERTY(alive_debug)) {
+			if (NODE_TALKS_G2(n)) {
+				g_debug("ALIVE dropped ping to %s", node_infostr(n));
+			} else {
+				g_debug("ALIVE dropped ping #%s to %s",
+					guid_hex_str(muid), node_infostr(n));
+			}
+		}
+
 		alive_ping_drop(a, muid);
 	}
 }
@@ -234,40 +266,54 @@ alive_pmsg_free(pmsg_t *mb, void *arg)
  * @return TRUE if we sent it, FALSE if there are too many ACK-pending pings.
  */
 bool
-alive_send_ping(void *obj)
+alive_send_ping(alive_t *a)
 {
-	struct alive *a = obj;
-	struct guid muid;
 	struct alive_ping *ap;
-	gnutella_msg_init_t *m;
-	uint32 size;
 	pmsg_t *mb;
 
-	g_assert(a->count == 0 || a->pings != NULL);
+	alive_check(a);
 
-	if (a->count >= a->maxcount)
+	if (eslist_count(&a->pings) >= a->maxcount)
 		return FALSE;
 
-	guid_ping_muid(&muid);
+	if (NODE_TALKS_G2(a->node)) {
+		pmsg_t *emb;
 
-	ap = ap_make(&muid);
-	a->count++;
-	a->pings = g_slist_append(a->pings, ap);
-	a->node->last_alive_ping = tm_time();
+		/*
+		 * G2 pings do not bear any MUID.
+		 */
 
-	/*
-	 * Build ping message and attach a pre-sender callback, as well as
-	 * a free routine to see whether the message is sent.
-	 */
+		ap = ap_make(NULL);
+		mb = g2_build_alive_ping();
+		emb = pmsg_clone_extend(mb, alive_pmsg_free, a->node);
+		pmsg_free(mb);
+		g2_node_send(a->node, emb);
+	} else {
+		struct guid muid;
+		gnutella_msg_init_t *m;
+		uint32 size;
 
-	m = build_ping_msg(&muid, 1, FALSE, &size);
+		guid_ping_muid(&muid);
 
-	g_assert(size == sizeof(*m));	/* No trailing GGEP extension */
+		ap = ap_make(&muid);
 
-	mb = gmsg_to_ctrl_pmsg_extend(m, size, alive_pmsg_free, a->node);
-	pmsg_set_check(mb, alive_ping_can_send);
+		/*
+		 * Build ping message and attach a pre-sender callback, as well as
+		 * a free routine to see whether the message is sent.
+		 */
 
-	gmsg_mb_sendto_one(a->node, mb);
+		m = build_ping_msg(&muid, 1, FALSE, &size);
+
+		g_assert(size == sizeof(*m));	/* No trailing GGEP extension */
+
+		mb = gmsg_to_ctrl_pmsg_extend(m, size, alive_pmsg_free, a->node);
+		pmsg_set_check(mb, alive_ping_can_send);
+
+		gmsg_mb_sendto_one(a->node, mb);
+	}
+
+	eslist_append(&a->pings, ap);
+	a->last_sent = tm_time();
 
 	return TRUE;
 }
@@ -307,30 +353,30 @@ ap_ack(const struct alive_ping *ap, struct alive *a)
 
 	a->avg_rt += (delay >> 2) - (a->avg_rt >> 2);
 
-	if (GNET_PROPERTY(alive_debug) > 1)
+	if (GNET_PROPERTY(alive_debug) > 1) {
 		g_debug("ALIVE %s "
-		"delay=%dms min=%dms, max=%dms, agv=%dms [%d queued]",
+			"delay=%dms min=%dms, max=%dms, agv=%dms [%zd queued]",
 			node_infostr(a->node),
-			a->last_rt, a->min_rt, a->max_rt, a->avg_rt, a->count);
+			a->last_rt, a->min_rt, a->max_rt, a->avg_rt,
+			eslist_count(&a->pings));
+	}
 }
 
 /**
- * Trim list of pings upto the specified linkable `item', included.
+ * Trim list of pings upto the specified `item', included.
  */
 static void
-alive_trim_upto(struct alive *a, GSList *item)
+alive_trim_upto(struct alive *a, struct alive_ping *item)
 {
-	GSList *sl;
-	bool found = FALSE;
+	struct alive_ping *ap;
 
-	g_assert(a->count && a->pings != NULL);
-
-	for (sl = a->pings; sl && !found; sl = a->pings) {
-		found = sl == item;
-		alive_remove_link(a, sl);
+	while (NULL != (ap = eslist_shift(&a->pings))) {
+		ap_free(ap);
+		if (ap == item)
+			return;
 	}
-	g_assert(found); /* Must have found the item */
-	g_assert(a->count >= 0);
+
+	g_assert(NULL == item);		/* Must have found the item if given */
 }
 
 /**
@@ -339,23 +385,37 @@ alive_trim_upto(struct alive *a, GSList *item)
  * @return TRUE if it was indeed an ACK for a ping we sent.
  */
 bool
-alive_ack_ping(void *obj, const struct guid *muid)
+alive_ack_ping(alive_t *a, const guid_t *muid)
 {
-	struct alive *a = obj;
-	GSList *sl;
+	struct alive_ping *ap;
 
-	g_assert(a->count == 0 || a->pings != NULL);
+	alive_check(a);
+	g_assert(!NODE_TALKS_G2(a->node) == (muid != NULL));
 
-	for (sl = a->pings; sl; sl = g_slist_next(sl)) {
-		const struct alive_ping *ap = sl->data;
+	if (NULL == muid) {
+		/*
+		 * G2 pong, no MUID so we only have one pending ping at a time.
+		 */
 
-		if (guid_eq(ap->muid, muid)) {		/* Found it! */
-			if (GNET_PROPERTY(alive_debug))
-				g_debug("ALIVE got alive pong #%s from %s",
-					guid_hex_str(muid), node_infostr(a->node));
+		ap = eslist_shift(&a->pings);
+		if (ap != NULL) {
+			if (GNET_PROPERTY(alive_debug)) {
+				g_debug("ALIVE got alive pong from %s", node_infostr(a->node));
+			}
 			ap_ack(ap, a);
-			alive_trim_upto(a, sl);
-			return TRUE;
+			alive_trim_upto(a, NULL);		/* Paranoid: remove everything */
+		}
+	} else {
+		ESLIST_FOREACH_DATA(&a->pings, ap) {
+			if (guid_eq(ap->muid, muid)) {		/* Found it! */
+				if (GNET_PROPERTY(alive_debug)) {
+					g_debug("ALIVE got alive pong #%s from %s",
+						guid_hex_str(muid), node_infostr(a->node));
+				}
+				ap_ack(ap, a);
+				alive_trim_upto(a, ap);
+				return TRUE;
+			}
 		}
 	}
 
@@ -367,38 +427,34 @@ alive_ack_ping(void *obj, const struct guid *muid)
  * replying to an alive ping.
  */
 void
-alive_ack_first(void *obj, const struct guid *muid)
+alive_ack_first(alive_t *a, const struct guid *muid)
 {
-	struct alive *a = obj;
 	struct alive_ping *ap;
-	GSList *sl;
 
-	g_assert(a->count == 0 || a->pings != NULL);
+	alive_check(a);
 
 	/*
 	 * Maybe they're reusing the same MUID we used for our "alive ping"?
 	 */
 
-	if (alive_ack_ping(obj, muid))
+	if (alive_ack_ping(a, muid))
 		return;
 
 	/*
 	 * Assume they're replying to the first ping we sent, ignore otherwise.
 	 */
 
-	sl = a->pings;		/* First ping we sent, chronologically */
+	ap = eslist_shift(&a->pings);	/* First ping we sent, chronologically */
 
-	if (NULL == sl)
+	if (NULL == ap)
 		return;
 
-	ap = sl->data;
-
 	if (GNET_PROPERTY(alive_debug))
 		g_debug("ALIVE TTL=0 ping from %s, assuming it acks #%s",
 			node_infostr(a->node), guid_hex_str(ap->muid));
 
 	ap_ack(ap, a);
-	alive_remove_link(a, sl);
+	ap_free(ap);
 }
 
 /**
@@ -406,14 +462,23 @@ alive_ack_first(void *obj, const struct guid *muid)
  * Values are expressed in milliseconds.
  */
 void
-alive_get_roundtrip_ms(const void *obj, uint32 *avg, uint32 *last)
+alive_get_roundtrip_ms(const alive_t *a, uint32 *avg, uint32 *last)
 {
-	const struct alive *a = obj;
-
-	g_assert(NULL != obj);
+	alive_check(a);
 
 	if (avg)	*avg = a->avg_rt;
 	if (last)	*last = a->last_rt;
 }
 
+/**
+ * @return how much time elapsed since the last alive ping.
+ */
+time_delta_t
+alive_elapsed(const alive_t *a)
+{
+	alive_check(a);
+
+	return delta_time(tm_time(), a->last_sent);
+}
+
 /* vi: set ts=4 sw=4 cindent: */
