@@ -39,6 +39,7 @@
 
 #include "lib/endian.h"
 #include "lib/mempcpy.h"
+#include "lib/stringify.h"
 #include "lib/walloc.h"
 
 #include "lib/override.h"
@@ -100,18 +101,18 @@ dime_record_free(struct dime_record **record_ptr)
 }
 
 void
-dime_list_free(GSList **list_ptr)
+dime_list_free(pslist_t **list_ptr)
 {
-	GSList *list = *list_ptr;
+	pslist_t *list = *list_ptr;
 
 	if (list) {
-		GSList *iter;
+		pslist_t *iter;
 
-		for (iter = list; NULL != iter; iter = g_slist_next(iter)) {
+		PSLIST_FOREACH(list, iter) {
 			struct dime_record *record = iter->data;
 			dime_record_free(&record);
 		}
-		g_slist_free(list);
+		pslist_free(list);
 		*list_ptr = NULL;
 	}
 }
@@ -230,7 +231,7 @@ dime_log_truncated_record(const char *name, const struct dime_record *header,
 		(header->flags & DIME_F_MB) ? " [MB]" : "",
 		(header->flags & DIME_F_ME) ? " [ME]" : "",
 		(header->flags & DIME_F_CF) ? " [CF]" : "",
-		announced, dime_ceil(announced), real, 1 == real ? "" : "s");
+		announced, dime_ceil(announced), real, plural(real));
 }
 
 /**
@@ -258,9 +259,9 @@ dime_parse_record_header(const char *data, size_t size,
 	header->version = peek_u8(&data[0]) >> 3;
 
 	if (DIME_VERSION != header->version) {
-		g_warning("dime_parse_record_header(): Cannot parse dime version %u, "
+		g_warning("%s(): cannot parse dime version %u, "
 			"only version %u is supported",
-			header->version, DIME_VERSION);
+			G_STRFUNC, header->version, DIME_VERSION);
 		goto failure;
 	}
 
@@ -319,18 +320,18 @@ failure:
 	return 0;
 }
 
-GSList *
+pslist_t *
 dime_parse_records(const char *data, size_t size)
 {
 	const char * const data0 = data;
-	GSList *list = NULL;
+	pslist_t *list = NULL;
 
 	for (;;) {
 		struct dime_record *record;
 		size_t ret;
 		
 		record = dime_record_alloc();
-		list = g_slist_prepend(list, record);
+		list = pslist_prepend(list, record);
 		
 		ret = dime_parse_record_header(data, size, record);
 		if (0 == ret) {
@@ -350,7 +351,7 @@ dime_parse_records(const char *data, size_t size)
 		}
 	}
 
-	return g_slist_reverse(list);
+	return pslist_reverse(list);
 
 error:
 
