@@ -120,12 +120,13 @@ int dmesh_count(const struct sha1 *sha1);
 
 bool dmesh_collect_sha1(const char *value, struct sha1 *sha1);
 void dmesh_collect_locations(const struct sha1 *sha1, const char *value,
-	const gnet_host_t *origin);
+	const gnet_host_t *origin, const char *user_agent);
 void dmesh_collect_compact_locations(const struct sha1 *sha1,
-		const char *value, const gnet_host_t *origin);
-void dmesh_collect_fw_hosts(const struct sha1 *sha1, const char *value);
+		const char *value, const gnet_host_t *origin, const char *user_agent);
+void dmesh_collect_fw_hosts(const struct sha1 *sha1, const char *value,
+	const gnet_host_t *origin, const char *user_agent);
 void dmesh_collect_negative_locations(const struct sha1 *sha1,
-	const char *value, host_addr_t reporter);
+	const char *value, host_addr_t reporter, const char *user_agent);
 int dmesh_fill_alternate(const struct sha1 *sha1,
 		gnet_host_t *hvec, int hcnt);
 
