@@ -37,20 +37,23 @@
 #include <math.h>	/* For pow() */
 #endif	/* I_MATH */
 
-#include "dq.h"
-#include "gmsg.h"
-#include "gmsg.h"
-#include "nodes.h"
-#include "gnet_stats.h"
-#include "qrp.h"
-#include "vmsg.h"
+#define SEARCH_SOURCES
 #include "search.h"
+
+#include "dq.h"
+
 #include "alive.h"
-#include "oob_proxy.h"
-#include "sockets.h"		/* For udp_active() */
-#include "settings.h"		/* For listen_addr() */
+#include "gmsg.h"
+#include "gmsg.h"
+#include "gnet_stats.h"
 #include "hosts.h"			/* For host_is_valid() */
+#include "nodes.h"
+#include "oob_proxy.h"
+#include "qrp.h"
+#include "settings.h"		/* For listen_addr() */
 #include "share.h"			/* For query_strip_oob_flag() */
+#include "sockets.h"		/* For udp_active() */
+#include "vmsg.h"
 
 #include "if/gnet_property_priv.h"
 
@@ -64,6 +67,7 @@
 #include "lib/hset.h"
 #include "lib/htable.h"
 #include "lib/nid.h"
+#include "lib/pslist.h"
 #include "lib/stringify.h"
 #include "lib/tm.h"
 #include "lib/vsort.h"
@@ -194,7 +198,7 @@ static hevset_t *dqueries;
 /**
  * This table keeps track of all the dynamic query objects created
  * for a given node ID.  The key is the node ID (converted to a pointer) and
- * the value is a GSList containing all the queries for that node.
+ * the value is a pslist_t containing all the queries for that node.
  */
 static htable_t *by_node_id;
 
@@ -559,7 +563,7 @@ dq_pmsg_free(pmsg_t *mb, void *arg)
 				nid_to_string(&dq->qid),
 				node_id_self(dq->node_id) ? "local " : "",
 				(int) (tm_time() - dq->start),
-				dq->up_sent, dq->up_sent == 1 ? "" :"s",
+				dq->up_sent, plural(dq->up_sent),
 				dq->horizon, dq->results);
 		}
 	}
@@ -631,13 +635,13 @@ dq_pmsg_by_ttl(dquery_t *dq, int ttl)
 static int
 dq_fill_probe_up(dquery_t *dq, gnutella_node_t **nv, int ncount)
 {
-	const GSList *sl;
+	const pslist_t *sl;
 	int i = 0;
 
 	dquery_check(dq);
 
-	GM_SLIST_FOREACH(node_all_ultranodes(), sl) {
-		struct gnutella_node *n;
+	PSLIST_FOREACH(node_all_ultranodes(), sl) {
+		gnutella_node_t *n;
 
 		if (i >= ncount)
 			break;
@@ -726,7 +730,7 @@ dq_free_next_up(dquery_t *dq)
 static int
 dq_fill_next_up(dquery_t *dq, struct next_up *nv, int ncount)
 {
-	const GSList *sl;
+	const pslist_t *sl;
 	int i = 0;
 	htable_t *old = NULL;
 
@@ -753,9 +757,9 @@ dq_fill_next_up(dquery_t *dq, struct next_up *nv, int ncount)
 	 * Select candidate ultra peers for sending query.
 	 */
 
-	GM_SLIST_FOREACH(node_all_ultranodes(), sl) {
+	PSLIST_FOREACH(node_all_ultranodes(), sl) {
 		struct next_up *nup, *old_nup;
-		struct gnutella_node *n;
+		gnutella_node_t *n;
 		const void *knid;
 		void *ttlv;
 		bool found;
@@ -852,7 +856,7 @@ static void
 dq_sendto_leaves(dquery_t *dq, gnutella_node_t *source)
 {
 	const void *head;
-	GSList *nodes;
+	pslist_t *nodes;
 
 	dquery_check(dq);
 	head = cast_to_constpointer(pmsg_start(dq->mb));
@@ -867,14 +871,14 @@ dq_sendto_leaves(dquery_t *dq, gnutella_node_t *source)
 				gnutella_header_get_hops(head), 0, TRUE, source);
 
 	if (GNET_PROPERTY(dq_debug) > 4)
-		g_debug("DQ QRP %s (%d word%s%s) forwarded to %d/%d leaves",
+		g_debug("DQ QRP %s (%d word%s%s) forwarded to %zd/%d leaves",
 			gmsg_infostr_full(head, pmsg_written_size(dq->mb)),
-			qhvec_count(dq->qhv), qhvec_count(dq->qhv) == 1 ? "" : "s",
+			qhvec_count(dq->qhv), plural(qhvec_count(dq->qhv)),
 			qhvec_has_urn(dq->qhv) ? " + URN" : "",
-			g_slist_length(nodes), GNET_PROPERTY(node_leaf_count));
+			pslist_length(nodes), GNET_PROPERTY(node_leaf_count));
 
 	gmsg_mb_sendto_all(nodes, dq->mb);
-	g_slist_free(nodes);
+	pslist_free(nodes);
 }
 
 static void
@@ -970,16 +974,16 @@ dq_free(dquery_t *dq)
 	if (!node_id_self(dq->node_id) && !(dq->flags & DQ_F_ID_CLEANING)) {
 		void *value;
 		bool found;
-		GSList *list;
+		pslist_t *list;
 
 		found = htable_lookup_extended(by_node_id, dq->node_id, NULL, &value);
 
 		if (!found) {
-			g_error("%s: missing %s", G_STRLOC, nid_to_string(dq->node_id));
+			g_error("%s: missing %s", G_STRFUNC, nid_to_string(dq->node_id));
 		}
 
 		list = value;
-		list = g_slist_remove(list, dq);
+		list = pslist_remove(list, dq);
 
 		if (list == NULL) {
 			/* Last item removed, get rid of the entry */
@@ -1039,19 +1043,17 @@ dq_free(dquery_t *dq)
  * Callout queue callback invoked when the dynamic query has expired.
  */
 static void
-dq_expired(cqueue_t *unused_cq, void *obj)
+dq_expired(cqueue_t *cq, void *obj)
 {
 	dquery_t *dq = obj;
 
 	dquery_check(dq);
 	g_assert(dq->expire_ev != NULL);
 
-	(void) unused_cq;
-
 	if (GNET_PROPERTY(dq_debug) > 3)
 		g_debug("DQ[%s] expired", nid_to_string(&dq->qid));
 
-	dq->expire_ev = NULL;	/* Indicates callback fired */
+	cq_zero(cq, &dq->expire_ev);	/* Indicates callback fired */
 
 	/*
 	 * If query was lingering, free it.
@@ -1075,7 +1077,7 @@ dq_expired(cqueue_t *unused_cq, void *obj)
  * Callout queue callback invoked when the result timer has expired.
  */
 static void
-dq_results_expired(cqueue_t *unused_cq, void *obj)
+dq_results_expired(cqueue_t *cq, void *obj)
 {
 	dquery_t *dq = obj;
 	gnutella_node_t *n;
@@ -1084,12 +1086,11 @@ dq_results_expired(cqueue_t *unused_cq, void *obj)
 	uint32 last;
 	bool was_waiting = FALSE;
 
-	(void) unused_cq;
 	dquery_check(dq);
 	g_assert(!(dq->flags & DQ_F_LINGER));
 	g_assert(dq->results_ev != NULL);
 
-	dq->results_ev = NULL;	/* Indicates callback fired */
+	cq_zero(cq, &dq->results_ev);	/* Indicates callback fired */
 
 	/*
 	 * If we were waiting for a status reply from the queryier, well, we
@@ -1140,8 +1141,7 @@ dq_results_expired(cqueue_t *unused_cq, void *obj)
 		if (GNET_PROPERTY(dq_debug) > 1)
 			g_debug(
 				"DQ[%s] terminating unguided & unrouted (queried %u UP%s)",
-				nid_to_string(&dq->qid), dq->up_sent,
-				dq->up_sent == 1 ? "" : "s");
+				nid_to_string(&dq->qid), dq->up_sent, plural(dq->up_sent));
 		dq_terminate(dq);
 		return;
 	}
@@ -1494,7 +1494,7 @@ dq_send_next(dquery_t *dq)
 
 	if (GNET_PROPERTY(dq_debug) > 19)
 		g_debug("DQ[%s] still %d UP%s to query (results %sso far: %u)",
-			nid_to_string(&dq->qid), found, found == 1 ? "" : "s",
+			nid_to_string(&dq->qid), found, plural(found),
 			(dq->flags & DQ_F_GOT_GUIDANCE) ? "reported kept " : "", results);
 
 	if (found == 0)
@@ -1516,7 +1516,7 @@ dq_send_next(dquery_t *dq)
 	 */
 
 	for (i = 0; i < found; i++) {
-		struct gnutella_node *node;
+		gnutella_node_t *node;
 		struct nid *nid = nv[i].node_id;
 		const void *knid;
 		void *ttlv;
@@ -1633,7 +1633,7 @@ dq_send_probe(dquery_t *dq)
 
 	if (GNET_PROPERTY(dq_debug) > 19)
 		g_debug("DQ[%s] found %d UP%s to probe",
-			nid_to_string(&dq->qid), found, found == 1 ? "" : "s");
+			nid_to_string(&dq->qid), found, plural(found));
 
 	/*
 	 * If we don't find any suitable UP holding that content, then
@@ -1733,16 +1733,16 @@ dq_common_init(dquery_t *dq)
 
 	if (!(dq->flags & DQ_F_LOCAL)) {
 		bool found;
-		GSList *list;
+		pslist_t *list;
 
 		found = htable_lookup_extended(by_node_id, dq->node_id, NULL, &value);
 
 		if (found) {
 			list = value;
-			list = gm_slist_insert_after(list, list, dq);
+			list = pslist_insert_after(list, list, dq);
 			g_assert(list == value);		/* Head not changed */
 		} else {
-			list = g_slist_prepend(NULL, dq);
+			list = pslist_prepend(NULL, dq);
 			htable_insert(by_node_id, dq->node_id, list);
 			g_assert(htable_lookup(by_node_id, dq->node_id) == list);
 		}
@@ -1762,7 +1762,7 @@ dq_common_init(dquery_t *dq)
 			"already used by %s.",
 			guid_hex_str(muid), node_id_infostr(dq->node_id),
 			dq->node_id == odq->node_id ?
-				"same node" : node_id_infostr(odq->node_id));
+				"same node" : node_id_infostr2(odq->node_id));
 	} else {
 		htable_insert(by_muid, atom_guid_get(muid), dq);
 	}
@@ -1781,7 +1781,7 @@ dq_common_init(dquery_t *dq)
 				"dynamic query from %s, already used by %s",
 				guid_hex_str(dq->lmuid), node_id_infostr(dq->node_id),
 				dq->node_id == odq->node_id ?
-					"same node" : node_id_infostr(odq->node_id));
+					"same node" : node_id_infostr2(odq->node_id));
 		} else {
 			hikset_insert_key(by_leaf_muid, &dq->lmuid);
 		}
@@ -1821,25 +1821,30 @@ dq_common_init(dquery_t *dq)
  *
  * @param n				leaf node from which query comes from
  * @param qhv			computed query hash vector, for routing query via QRT
- * @param media_types	requested media type filters (0 if none)
+ * @param sri			query information from pre-processing stage
  */
 void
-dq_launch_net(gnutella_node_t *n, query_hashvec_t *qhv, unsigned media_types)
+dq_launch_net(
+	gnutella_node_t *n,
+	query_hashvec_t *qhv, const search_request_info_t *sri)
 {
 	dquery_t *dq;
 	uint16 flags;
-	bool flags_valid;
-	const struct guid *leaf_muid;
+	bool flags_valid, proxied = FALSE, need_proxying = FALSE;
+	const guid_t *leaf_muid;
+	guid_t orig_muid;
 
 	/* Query from leaf node */
 	g_assert(NODE_IS_LEAF(n));
 	g_assert(gnutella_header_get_hops(&n->header) == 1);
 	g_assert(NODE_IS_CONNECTED(n));
 
+	memcpy(&orig_muid, gnutella_header_get_muid(&n->header), GUID_RAW_SIZE);
+
 	WALLOC0(dq);
 	dq->magic = DQUERY_MAGIC;
 
-	flags = peek_be16(n->data);
+	flags = sri->flags;
 	flags_valid = 0 != (flags & QUERY_F_MARK);
 
 	/*
@@ -1862,70 +1867,155 @@ dq_launch_net(gnutella_node_t *n, query_hashvec_t *qhv, unsigned media_types)
 		dq->flags |= DQ_F_LEAF_GUIDED;
 
 	/*
-	 * If the query is not leaf-guided and not OOB proxied already, then we
-	 * need to ensure results are routed back to us.
+	 * If the query is not leaf-guided, then we need to ensure results are
+	 * routed back to us by OOB-proxying the query.
+	 *
 	 * We won't know how much they filter out however, but they just have
 	 * to implement proper leaf-guidance for better results as leaves...
 	 *		--RAM, 2006-08-16
+	 *
+	 * If the leaf node is firewalled and won't be able to get UDP hits back,
+	 * make sure we also OOB-proxy the query (even if they support leaf
+	 * guidance) to avoid extra TCP traffic from the neighbouring ultrapeers.
+	 *
+	 * If the leaf node did not request OOB results, we also OOB-proxy it,
+	 * again to avoid extra TCP relaying from neighbouring ultrapeers.
+	 *
+	 * Because gtk-gnutella supports OOB-v3, we do not honour the no-proxying
+	 * flag that the host could send us.  This indication was devised by
+	 * LimeWire before the OOB-v3 days and is considered obsolete and useless
+	 * nowadays.
+	 *		--RAM, 2014-03-10
 	 */
 
-	if (
-		!(dq->flags & DQ_F_LEAF_GUIDED) &&
-		NULL == oob_proxy_muid_proxied(gnutella_header_get_muid(&n->header))
-	) {
-		bool proxied = FALSE;
-		if (
-			!GNET_PROPERTY(is_udp_firewalled) &&
-			GNET_PROPERTY(proxy_oob_queries) &&
-			udp_active() &&
-			host_is_valid(listen_addr(), socket_listen_port())
-			/* NOTE: IPv6 OOB proxying won't work, so don't check for IPv6 */
-		) {
-			/*
-			 * Running with UDP support.
-			 * OOB-proxy the query so that we can control how much results
-			 * they get by routing the results ourselves to the leaf.
-			 */
+	need_proxying = !(dq->flags & DQ_F_LEAF_GUIDED) ||
+		(flags & QUERY_F_FIREWALLED) || !sri->oob;
 
-			if (GNET_PROPERTY(dq_debug) > 19)
-				g_debug("DQ %s #%s OOB-proxying query \"%s\" (%s)",
+	if (
+		need_proxying &&
+		!GNET_PROPERTY(is_udp_firewalled) &&
+		GNET_PROPERTY(proxy_oob_queries) &&
+		udp_active() &&
+		host_is_valid(listen_addr(), socket_listen_port())
+		/* NOTE: IPv6 OOB proxying won't work, so don't check for IPv6 */
+	) {
+		/*
+		 * Running with UDP support.
+		 * OOB-proxy the query so that we can control how much results
+		 * they get by routing the results ourselves to the leaf.
+		 */
+
+		if (GNET_PROPERTY(dq_debug) > 3) {
+			g_debug("DQ %s #%s OOB-proxying query \"%s\" (%s)",
+				node_infostr(n), nid_to_string(NODE_ID(n)),
+				n->data + 2,
+				(flags_valid && (flags & QUERY_F_LEAF_GUIDED)) ?
+					"guided" : "unguided"
+			);
+		}
+
+		if (oob_proxy_create(n)) {
+			proxied = TRUE;
+		} else {
+			if (GNET_PROPERTY(dq_debug)) {
+				g_warning("DQ %s #%s: "
+					"cannot OOB-proxy query \"%s\" (%s): MUID collision",
 					node_infostr(n), nid_to_string(NODE_ID(n)),
 					n->data + 2,
 					(flags_valid && (flags & QUERY_F_LEAF_GUIDED)) ?
-						"guided" : "unguided"
-				);
-
-			if (oob_proxy_create(n)) {
-				gnet_stats_inc_general(GNR_OOB_PROXIED_QUERIES);
-				proxied = TRUE;
-			} else {
-				if (GNET_PROPERTY(dq_debug)) {
-					g_warning("DQ %s #%s: "
-						"cannot OOB-proxy query \"%s\" (%s): MUID collision",
-						node_infostr(n), nid_to_string(NODE_ID(n)),
-						n->data + 2,
-						(flags_valid && (flags & QUERY_F_LEAF_GUIDED)) ?
-							"guided" : "unguided");
-				}
+						"guided" : "unguided");
 			}
 		}
-		if (!proxied && flags_valid && (flags & QUERY_F_OOB_REPLY)) {
-			/*
-			 * Running without UDP support, or UDP-firewalled...
-			 * Must remove the OOB flag so that results be routed back.
-			 */
+	}
 
-			query_strip_oob_flag(n, n->data);
-			flags = peek_be16(n->data);	/* Refresh our cache */
+	if (!proxied && sri->oob) {
+		/*
+		 * Running without UDP support, or UDP-firewalled...
+		 * Must remove the OOB flag so that results be routed back.
+		 */
 
-			if (GNET_PROPERTY(dq_debug) > 19)
-				g_debug(
-					"DQ %s #%s stripped OOB on query \"%s\" (%s)",
-					node_infostr(n), nid_to_string(NODE_ID(n)),
-					n->data + 2,
-					(flags_valid && (flags & QUERY_F_LEAF_GUIDED)) ?
-						"guided" : "unguided"
-				);
+		query_strip_oob_flag(n, n->data);
+		flags = peek_be16(n->data);	/* Refresh our cache */
+
+		if (GNET_PROPERTY(dq_debug) > 1) {
+			g_debug(
+				"DQ %s #%s stripped OOB on query \"%s\" (%s)",
+				node_infostr(n), nid_to_string(NODE_ID(n)),
+				n->data + 2,
+				(flags_valid && (flags & QUERY_F_LEAF_GUIDED)) ?
+					"guided" : "unguided"
+			);
+		}
+	}
+
+	/*
+	 * The so-called leaf MUID is the MUID of the query originally isssued by
+	 * the leaf node if the query is being OOB-proxied locally.  It will be
+	 * NULL if the query is not being OOB-proxied.
+	 *
+	 * When non-NULL, it must not be already known (associated to another
+	 * running DQ).
+	 */
+
+	leaf_muid = oob_proxy_muid_proxied(gnutella_header_get_muid(&n->header));
+
+	if (leaf_muid != NULL) {
+		dquery_t *odq = hikset_lookup(by_leaf_muid, leaf_muid);
+
+		if (odq != NULL) {
+			dquery_check(odq);
+
+			if (odq->flags & DQ_F_LINGER) {
+				/* No longer active, can remove this association */
+				hikset_remove(by_leaf_muid, leaf_muid);
+			} else {
+				g_warning("leaf MUID \"%s\" for dynamic query from %s, "
+					"would OOB-proxy to same MUID as \"%s\" already used by %s"
+					" -- dropping",
+					guid_hex_str(&orig_muid), node_infostr(n),
+					guid_to_string(leaf_muid),
+					NODE_ID(n) == odq->node_id ?
+						"same node" : node_id_infostr(odq->node_id));
+				goto oob_proxy_conflict;
+			}
+		}
+	}
+
+	/*
+	 * Likewise, the query MUID must not already be known.
+	 *
+	 * If the query MUID is not being OOB-proxied, it should already have
+	 * been caught by the routing table (as a duplicate incoming query).
+	 */
+
+	{
+		const guid_t *muid = gnutella_header_get_muid(&n->header);
+		dquery_t *odq = htable_lookup(by_muid, muid);
+
+		if (odq != NULL) {
+			dquery_check(odq);
+
+			if (odq->flags & DQ_F_LINGER) {
+				const void *key;
+				const guid_t *muid_key;
+				bool found;
+
+				found = htable_lookup_extended(by_muid, muid, &key, NULL);
+				g_assert(found);
+
+				/* No longer active, can remove this association */
+				htable_remove(by_muid, muid);
+				muid_key = key;
+				atom_guid_free_null(&muid_key);
+			} else {
+				g_warning("OOB-proxied MUID \"%s\" for dynamic query from %s, "
+					"is already used by another active query from %s"
+					" -- dropping",
+					guid_hex_str(muid), node_infostr(n),
+					NODE_ID(n) == odq->node_id ?
+						"same node" : node_id_infostr(odq->node_id));
+				goto oob_proxy_conflict;
+			}
 		}
 	}
 
@@ -1933,18 +2023,45 @@ dq_launch_net(gnutella_node_t *n, query_hashvec_t *qhv, unsigned media_types)
 	 * See whether we'll be seeing all the hits...
 	 */
 
-	if (
-		NULL != oob_proxy_muid_proxied(gnutella_header_get_muid(&n->header)) ||	
-		(flags_valid && !(flags & QUERY_F_OOB_REPLY))
-	)
+	if (NULL != leaf_muid || (flags_valid && !(flags & QUERY_F_OOB_REPLY)))
 		dq->flags |= DQ_F_ROUTING_HITS;
 
+	/*
+	 * We're going to launch this query coming from our leaf.
+	 */
+
+	if (proxied) {
+		gnet_stats_inc_general(GNR_OOB_PROXIED_QUERIES);
+
+		/*
+		 * We're supporting OOBv3, so make sure the "SO" key is present in
+		 * the query if we're OOB-proxying it.
+		 */
+
+		n->msg_flags &= ~NODE_M_STRIP_GE_SO;
+		n->msg_flags |= NODE_M_ADD_GE_SO | NODE_M_EXT_CLEANUP;
+
+		/*
+		 * If they did not want us to OOB-proxy their query, emit a small
+		 * warning when debugging.
+		 */
+
+		if (NODE_NO_OOB_PROXY(n)) {
+			if (GNET_PROPERTY(dq_debug) || GNET_PROPERTY(oob_proxy_debug)) {
+				g_warning("OOB-proxied MUID \"%s\" for dynamic query from %s, "
+					"despite host vetoing it",
+					guid_to_string(leaf_muid), node_infostr(n));
+			}
+		}
+	}
+
 	/*
 	 * Compact query if requested.
 	 */
 
 	if (
-		GNET_PROPERTY(gnet_compact_query) || (n->msg_flags & NODE_M_EXT_CLEANUP)
+		GNET_PROPERTY(gnet_compact_query) ||
+		(n->msg_flags & NODE_M_EXT_CLEANUP)
 	)
 		search_compact(n);
 
@@ -1961,7 +2078,6 @@ dq_launch_net(gnutella_node_t *n, query_hashvec_t *qhv, unsigned media_types)
 	if (flags_valid)
 		dq->query_flags = flags;
 
-	leaf_muid = oob_proxy_muid_proxied(gnutella_header_get_muid(&n->header));
 	if (leaf_muid != NULL)
 		dq->lmuid = atom_guid_get(leaf_muid);
 
@@ -1970,7 +2086,7 @@ dq_launch_net(gnutella_node_t *n, query_hashvec_t *qhv, unsigned media_types)
 
 		packet = pmsg_start(dq->mb);
 		record_query_string(gnutella_header_get_muid(packet),
-			gnutella_msg_search_get_text(packet), media_types);
+			gnutella_msg_search_get_text(packet), sri->media_types);
 	}
 
 	if (GNET_PROPERTY(dq_debug) > 1) {
@@ -1993,6 +2109,19 @@ dq_launch_net(gnutella_node_t *n, query_hashvec_t *qhv, unsigned media_types)
 	dq_common_init(dq);
 	dq_sendto_leaves(dq, n);
 	dq_send_probe(dq);
+	return;
+
+oob_proxy_conflict:
+	gnet_stats_count_dropped(n, MSG_DROP_OOB_PROXY_CONFLICT);
+
+	/*
+	 * Do not launch this dynamic query.  Since we have not yet created any
+	 * data structure when we're re-routed here, we can free up local DQ and
+	 * return.
+	 */
+
+	dq->magic = 0;
+	WFREE(dq);
 }
 
 /**
@@ -2084,7 +2213,7 @@ void
 dq_node_removed(const struct nid *node_id)
 {
 	void *value;
-	GSList *sl;
+	pslist_t *sl;
 
 	if (!htable_lookup_extended(by_node_id, node_id, NULL, &value))
 		return;		/* No dynamic query for this node */
@@ -2092,7 +2221,7 @@ dq_node_removed(const struct nid *node_id)
 	htable_remove(by_node_id, node_id);
 	g_assert(!htable_contains(by_node_id, node_id));
 
-	GM_SLIST_FOREACH(value, sl) {
+	PSLIST_FOREACH(value, sl) {
 		dquery_t *dq = sl->data;
 
 		dquery_check(dq);
@@ -2100,7 +2229,7 @@ dq_node_removed(const struct nid *node_id)
 		if (GNET_PROPERTY(dq_debug) > 1)
 			g_debug("DQ[%s] terminated by node #%s removal (queried %u UP%s)",
 				nid_to_string(&dq->qid), nid_to_string2(dq->node_id),
-				dq->up_sent, dq->up_sent == 1 ? "" : "s");
+				dq->up_sent, plural(dq->up_sent));
 		
 		/* Don't remove query from the table in dq_free() */
 		dq->flags |= DQ_F_ID_CLEANING;
@@ -2108,7 +2237,7 @@ dq_node_removed(const struct nid *node_id)
 	}
 
 	g_assert(!htable_contains(by_node_id, node_id));
-	g_slist_free(value);
+	pslist_free(value);
 }
 
 /**
@@ -2359,8 +2488,7 @@ dq_got_query_status(const struct guid *muid,
 	if (kept == 0xffff) {
 		if (GNET_PROPERTY(dq_debug) > 1)
 			g_debug("DQ[%s] terminating at user's request (queried %u UP%s)",
-				nid_to_string(&dq->qid), dq->up_sent,
-				dq->up_sent == 1 ? "" : "s");
+				nid_to_string(&dq->qid), dq->up_sent, plural(dq->up_sent));
 
 		dq->flags |= DQ_F_USR_CANCELLED;
 
@@ -2388,7 +2516,7 @@ dq_got_query_status(const struct guid *muid,
 
 struct cancel_context {
 	gnet_search_t handle;
-	GSList *cancelled;
+	pslist_t *cancelled;
 };
 
 /**
@@ -2404,7 +2532,7 @@ dq_cancel_local(void *value, void *udata)
 	dquery_check(dq);
 
 	if ((dq->flags & DQ_F_LOCAL) && dq->sh == ctx->handle) {
-		ctx->cancelled = g_slist_prepend(ctx->cancelled, dq);
+		ctx->cancelled = pslist_prepend(ctx->cancelled, dq);
 	}
 }
 
@@ -2415,17 +2543,17 @@ void
 dq_search_closed(gnet_search_t handle)
 {
 	struct cancel_context ctx;
-	GSList *sl;
+	pslist_t *sl;
 
 	ctx.handle = handle;
 	ctx.cancelled = NULL;
 
 	hevset_foreach(dqueries, dq_cancel_local, &ctx);
 
-	GM_SLIST_FOREACH(ctx.cancelled, sl) {
+	PSLIST_FOREACH(ctx.cancelled, sl) {
 		dq_free(sl->data);
 	}
-	gm_slist_free_null(&ctx.cancelled);
+	pslist_free_null(&ctx.cancelled);
 }
 
 /**
@@ -2522,14 +2650,14 @@ free_query(void *value, void *unused_udata)
 static void
 free_query_list(const void *key, void *value, void *unused_udata)
 {
-	GSList *sl, *list = value;
-	int count = g_slist_length(list);
+	pslist_t *sl, *list = value;
+	int count = pslist_length(list);
 
 	(void) unused_udata;
 	g_warning("remained %d un-freed dynamic quer%s for node #%u",
-		count, count == 1 ? "y" : "ies", GPOINTER_TO_UINT(key));
+		count, plural_y(count), GPOINTER_TO_UINT(key));
 
-	GM_SLIST_FOREACH(list, sl) {
+	PSLIST_FOREACH(list, sl) {
 		dquery_t *dq = sl->data;
 
 		dquery_check(dq);
@@ -2538,7 +2666,7 @@ free_query_list(const void *key, void *value, void *unused_udata)
 		dq_free(dq);
 	}
 
-	g_slist_free(list);
+	pslist_free(list);
 }
 
 /**
