@@ -181,8 +181,9 @@ gnet_stats_init(void)
  * Generate a SHA1 digest of the supplied statistics.
  */
 static void
-gnet_stats_digest(sha1_t *digest, const gnet_stats_t *stats)
+gnet_stats_digest(sha1_t *digest, gnet_stats_t *stats)
 {
+	stats->general[GNR_STATS_DIGEST]++;		/* Ensure ever-changing SHA1 */
 	SHA1_COMPUTE(*stats, digest);
 }
 
@@ -194,6 +195,7 @@ gnet_stats_digest(sha1_t *digest, const gnet_stats_t *stats)
 void
 gnet_stats_tcp_digest(sha1_t *digest)
 {
+	gnet_stats_inc_general(GNR_STATS_TCP_DIGEST);
 	gnet_stats_digest(digest, &gnet_tcp_stats);
 }
 
@@ -205,6 +207,7 @@ gnet_stats_tcp_digest(sha1_t *digest)
 void
 gnet_stats_udp_digest(sha1_t *digest)
 {
+	gnet_stats_inc_general(GNR_STATS_UDP_DIGEST);
 	gnet_stats_digest(digest, &gnet_udp_stats);
 }
 
@@ -216,6 +219,7 @@ gnet_stats_udp_digest(sha1_t *digest)
 void
 gnet_stats_general_digest(sha1_t *digest)
 {
+	gnet_stats_inc_general(GNR_STATS_DIGEST);
 	SHA1_COMPUTE(gnet_stats.general, digest);
 }
 
