@@ -77,7 +77,7 @@ static aging_table_t *node_connects;
 
 /*
  * Avoid nodes being stuck helplessly due to completely stale caches.
- * @return TRUE if an UHC may be contact, FALSE if it's not permissable.
+ * @return TRUE if an UHC may be contacted, FALSE if it's not permissable.
  */
 static bool
 host_cache_allow_bypass(int which)
@@ -155,7 +155,11 @@ host_timer(void)
 	host_type_t htype;
 	bool empty_cache = FALSE, empty_g2_cache = FALSE;
 
-	if (in_shutdown || !GNET_PROPERTY(online_mode))
+	if (
+		in_shutdown ||
+		!GNET_PROPERTY(online_mode) ||
+		GNET_PROPERTY(net_buffer_shortage)
+	)
 		return;
 
 	max_nodes = node_outdegree();	/* Gnutella, depending on leaf/ultra mode */
