@@ -319,7 +319,7 @@ parse_and_append_cache_entry(char *line)
 		if (
 			*p != '\t' ||
 			(p - sha1_digest_ascii) != SHA1_BASE32_SIZE ||
-			SHA1_RAW_SIZE != base32_decode(sha1.data, sizeof sha1.data,
+			SHA1_RAW_SIZE != base32_decode(&sha1, sizeof sha1,
 								sha1_digest_ascii, SHA1_BASE32_SIZE)
 		) {
 			goto failure;
@@ -707,7 +707,7 @@ huge_sha1_extract32(const char *buf, size_t len, struct sha1 *sha1,
 	if (len != SHA1_BASE32_SIZE || huge_improbable_sha1(buf, len))
 		goto bad;
 
-	if (SHA1_RAW_SIZE != base32_decode(sha1->data, sizeof sha1->data, buf, len))
+	if (SHA1_RAW_SIZE != base32_decode(sha1, sizeof *sha1, buf, len))
 		goto bad;
 
 	/*
@@ -756,7 +756,7 @@ huge_tth_extract32(const char *buf, size_t len, struct tth *tth,
 	if (len != TTH_BASE32_SIZE)
 		goto bad;
 
-	if (TTH_RAW_SIZE != base32_decode(tth->data, sizeof tth->data, buf, len))
+	if (TTH_RAW_SIZE != base32_decode(tth, sizeof *tth, buf, len))
 		goto bad;
 
 	return TRUE;
@@ -828,10 +828,25 @@ huge_collect_locations(const sha1_t *sha1, const header_t *header,
 {
 	char *alt;
 	size_t len;
+	const char *user_agent;
 
 	g_return_if_fail(sha1);
 	g_return_if_fail(header);
 
+	/*
+	 * This code can be invoked on the download path (when we analyse
+	 * locations sent by the server) or on the upload path (when we
+	 * analyze those sent by the user to whom we are uploading).
+	 *
+	 * Therefore the user agent name can be held in the Server or
+	 * User-Agent header, depending.
+	 */
+
+	user_agent = header_get(header, "User-Agent");	/* Uploading */
+
+	if (NULL == user_agent)
+		user_agent = header_get(header, "Server");	/* Downloading */
+
 	alt = header_get(header, "X-Gnutella-Alternate-Location");
 
 	/*
@@ -846,7 +861,7 @@ huge_collect_locations(const sha1_t *sha1, const header_t *header,
 		alt = header_get(header, "Alt-Location");
 
 	if (alt != NULL) {
-		dmesh_collect_locations(sha1, alt, origin);
+		dmesh_collect_locations(sha1, alt, origin, user_agent);
 		return;
 	}
 
@@ -861,9 +876,9 @@ huge_collect_locations(const sha1_t *sha1, const header_t *header,
 		 */
 
 		if (huge_is_pure_xalt(alt, len))
-			dmesh_collect_compact_locations(sha1, alt, origin);
+			dmesh_collect_compact_locations(sha1, alt, origin, user_agent);
 		else
-			dmesh_collect_locations(sha1, alt, origin);
+			dmesh_collect_locations(sha1, alt, origin, user_agent);
     }
 
 	/*
@@ -873,7 +888,7 @@ huge_collect_locations(const sha1_t *sha1, const header_t *header,
 	alt = header_get(header, "X-Falt");
 
 	if (alt != NULL) {
-		dmesh_collect_fw_hosts(sha1, alt);
+		dmesh_collect_fw_hosts(sha1, alt, origin, user_agent);
 	}
 }
 
