@@ -36,26 +36,16 @@
 
 #include "common.h"
 
+#include "if/gen/ignore.h"
+
 void ignore_init(void);
 void ignore_close(void);
 
-enum ignore_val {
-	IGNORE_FALSE = 0,		/**< Do not ignore */
-	IGNORE_SHA1,			/**< Ignore because of SHA1 */
-	IGNORE_NAMESIZE,		/**< Ignore because of Name & Size */
-	IGNORE_LIBRARY,			/**< Ignore because SHA1 present in library */
-	IGNORE_HOSTILE,			/**< Ignore because IP address is hostile */
-	IGNORE_OURSELVES,		/**< Ignore because IP:port points to ourselves */
-	IGNORE_LIMIT,			/**< Ignore because of country limits */
-	IGNORE_SPAM				/**< Ignore because SHA1 is known spam */
-};
-
 struct sha1;
 
 void ignore_timer(time_t now);
-enum ignore_val ignore_is_requested(
+ignore_val_t ignore_is_requested(
 	const char *file, filesize_t size, const struct sha1 *sha1);
-const char *ignore_reason_to_string(enum ignore_val);
 
 void ignore_add_filesize(const char *file, filesize_t size);
 void ignore_add_sha1(const char *file, const struct sha1 *sha1);
