@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002-2003, Raphael Manfredi
+ * Copyright (c) 2002-2003, 2014 Raphael Manfredi
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -28,7 +28,7 @@
  * Message queues, common code between TCP and UDP sending stacks.
  *
  * @author Raphael Manfredi
- * @date 2002-2003
+ * @date 2002-2003, 2014
  */
 
 #include "common.h"
@@ -40,15 +40,17 @@
 #include "gmsg.h"
 #include "gnet_stats.h"
 
+#include "lib/array_util.h"
 #include "lib/cq.h"
-#include "lib/glib-missing.h"	/* For gm_list_free_null() */
 #include "lib/halloc.h"
 #include "lib/htable.h"
+#include "lib/plist.h"
 #include "lib/pmsg.h"
 #include "lib/str.h"
+#include "lib/stringify.h"		/* For plural() */
 #include "lib/unsigned.h"		/* For size_saturate_add() */
-#include "lib/vsort.h"
 #include "lib/walloc.h"
+#include "lib/xsort_data.h"
 
 #include "if/gnet_property_priv.h"
 
@@ -59,7 +61,7 @@
 static void qlink_free(mqueue_t *q);
 static void mq_update_flowc(mqueue_t *q);
 static bool make_room_header(
-	mqueue_t *q, char *header, uint prio, int needed, int *offset);
+	mqueue_t *q, const char *header, uint prio, int needed, int *offset);
 static void mq_swift_timer(cqueue_t *cq, void *obj);
 
 /**
@@ -158,7 +160,7 @@ mq_bio(const mqueue_t *q)
 	return tx_bio_source(q->tx_drv);
 }
 
-struct gnutella_node *
+gnutella_node_t *
 mq_node(const mqueue_t *q)
 {
 	mq_check_consistency(q);
@@ -208,8 +210,8 @@ mq_info(const mqueue_t *q)
 			(q->flags & MQ_DISCARD) ? " DISCARD" : "",
 			(q->flags & MQ_SWIFT) ? " SWIFT" : "",
 			(q->flags & MQ_WARNZONE) ? " WARNZONE" : "",
-			q->count, q->count == 1 ? "" : "s",
-			q->size, q->size == 1 ? "" : "s"
+			q->count, plural(q->count),
+			q->size, plural(q->size)
 		);
 	}
 
@@ -226,7 +228,7 @@ static htable_t *qown = NULL;
  * Add linkable into queue
  */
 static void
-mq_add_linkable(mqueue_t *q, GList *l)
+mq_add_linkable(mqueue_t *q, plist_t *l)
 {
 	mqueue_t *owner;
 
@@ -254,7 +256,7 @@ mq_add_linkable(mqueue_t *q, GList *l)
  * Remove linkable from queue
  */
 static void
-mq_remove_linkable(mqueue_t *q, GList *l)
+mq_remove_linkable(mqueue_t *q, plist_t *l)
 {
 	mqueue_t *owner;
 
@@ -292,7 +294,7 @@ mq_check_track(mqueue_t *q, int offset, const char *where, int line)
 	if (q->magic != MQ_MAGIC)
 		g_error("BUG: %s at %s:%d", mq_info(q), where, line);
 
-	qcount = g_list_length(q->qhead);
+	qcount = plist_length(q->qhead);
 	if (qcount != q->count)
 		g_error("BUG: "
 			"%s has wrong q->count of %d (counted %d in list) at %s:%d",
@@ -302,7 +304,7 @@ mq_check_track(mqueue_t *q, int offset, const char *where, int line)
 		return;
 
 	for (n = 0; n < q->qlink_count; n++) {
-		GList *item = q->qlink[n];
+		plist_t *item = q->qlink[n];
 		mqueue_t *owner;
 
 		if (item == NULL)
@@ -354,14 +356,14 @@ mq_check_track(mqueue_t *q, int offset, const char *where, int line)
 void
 mq_free(mqueue_t *q)
 {
-	GList *l;
+	plist_t *l;
 	int n;
 
 	mq_check_consistency(q);
 
 	tx_free(q->tx_drv);		/* Get rid of lower layers */
 
-	for (n = 0, l = q->qhead; l; l = g_list_next(l)) {
+	for (n = 0, l = q->qhead; l; l = plist_next(l)) {
 		n++;
 		pmsg_free(l->data);
 		l->data = NULL;
@@ -374,7 +376,7 @@ mq_free(mqueue_t *q)
 		qlink_free(q);
 
 	cq_cancel(&q->swift_ev);
-	gm_list_free_null(&q->qhead);
+	plist_free_null(&q->qhead);
 	pmsg_slist_free(&q->qwait);
 
 	q->magic = 0;
@@ -388,13 +390,13 @@ mq_free(mqueue_t *q)
  * The underlying message is freed and the size information on the
  * queue is updated, but not the flow-control information.
  */
-static GList *
-mq_rmlink_prev(mqueue_t *q, GList *l, int size)
+static plist_t *
+mq_rmlink_prev(mqueue_t *q, plist_t *l, int size)
 {
-	GList *prev = g_list_previous(l);
+	plist_t *prev = plist_prev(l);
 
 	mq_remove_linkable(q, l);
-	q->qhead = g_list_remove_link(q->qhead, l);
+	q->qhead = plist_remove_link(q->qhead, l);
 	if (q->qtail == l)
 		q->qtail = prev;
 
@@ -405,7 +407,7 @@ mq_rmlink_prev(mqueue_t *q, GList *l, int size)
 
 	pmsg_free(l->data);
 	l->data = NULL;
-	g_list_free_1(l);
+	plist_free_1(l);
 
 	return prev;
 }
@@ -489,69 +491,31 @@ mq_swift_checkpoint(mqueue_t *q, bool initial)
 		needed = extra + flushed_till_next_timer / 4;
 	}
 
-	if (initial) {
-		/*
-		 * First time we're in "swift" mode.
-		 *
-		 * Purge pending queries, since they are getting quite old.
-		 * Leave our queries in for now (they have hops=0).
-		 */
+	/*
+	 * To cut the dependency between MQ and the messages being held within,
+	 * things are decoupled thusly:
+	 *
+	 * The MQ requests, via a user-supplied callback, a vector of message
+	 * templates.  For instance for Gnutella messages. these are just Gnutella
+	 * headers.
+	 *
+	 * These headers are used in turn to request message pruning from the
+	 * queue by comparing messages held in the queue with these headers,
+	 * using the user-supplied ``msg_headcmp'' comparison callback.
+	 */
 
-		gnutella_header_set_function(&q->header, GTA_MSG_SEARCH);
-		gnutella_header_set_hops(&q->header, 1);
-		gnutella_header_set_ttl(&q->header, GNET_PROPERTY(max_ttl));
+	if (q->uops->msg_templates != NULL) {
+		size_t i, tcnt = 0;
+		iovec_t *templates = q->uops->msg_templates(initial, &tcnt);
 
-		if (needed > 0)
-			make_room_header(q, (char*) &q->header, PMSG_P_DATA, needed, NULL);
+		for (i = 0; i < tcnt && needed > 0; i++) {
+			int old_size = q->size;
+			const void *base = iovec_base(&templates[i]);
 
-		/*
-		 * Whether or not we were able to make enough room at this point
-		 * is not important, for the initial checkpoint.  Indeed, since
-		 * we are now in "swift" mode, more query messages will be dropped
-		 * at the next iteration, since we'll start dropping query hits,
-		 * and hits are more prioritary than queries.
-		 */
-
-	} else {
-		int ttl;
-
-		/*
-		 * We're going to drop query hits...
-		 *
-		 * We start with the lowest prioritary query hit: low hops count
-		 * and high TTL, and we progressively increase until we can drop
-		 * the amount we need to drop.
-		 *
-		 * Note that we will never be able to drop the partially written
-		 * message at the tail of the queue, even if it is less prioritary
-		 * than our comparison point.
-		 */
-
-		gnutella_header_set_function(&q->header, GTA_MSG_SEARCH_RESULTS);
-
-		/*
-		 * Loop until we reach hops=hard_ttl_limit or we have finished
-		 * removing enough data from the queue.
-		 */
-
-		for (ttl = GNET_PROPERTY(hard_ttl_limit); ttl >= 0; ttl--) {
-			int old_size;
-
-			if (needed <= 0)
+			if (make_room_header(q, base, PMSG_P_DATA, needed, NULL))
 				break;
 
-			old_size = q->size;
-			gnutella_header_set_hops(&q->header,
-				GNET_PROPERTY(hard_ttl_limit) - ttl);
-			gnutella_header_set_ttl(&q->header, ttl);
-
-			if (
-				make_room_header(q, (char*) &q->header, PMSG_P_DATA,
-					needed, NULL)
-			)
-				break;
-
-			needed -= (old_size - q->size);		/* Amount we removed */
+			needed -= old_size - q->size;		/* Amount we removed */
 		}
 	}
 
@@ -576,13 +540,13 @@ done:
  * Callout queue callback: periodic "swift" mode timer.
  */
 static void
-mq_swift_timer(cqueue_t *unused_cq, void *obj)
+mq_swift_timer(cqueue_t *cq, void *obj)
 {
 	mqueue_t *q = obj;
 
-	(void) unused_cq;
 	g_assert((q->flags & (MQ_FLOWC|MQ_SWIFT)) == (MQ_FLOWC|MQ_SWIFT));
 
+	cq_zero(cq, &q->swift_ev);
 	mq_swift_checkpoint(q, FALSE);
 }
 
@@ -590,15 +554,15 @@ mq_swift_timer(cqueue_t *unused_cq, void *obj)
  * Callout queue callback invoked when the queue must enter "swift" mode.
  */
 static void
-mq_enter_swift(cqueue_t *unused_cq, void *obj)
+mq_enter_swift(cqueue_t *cq, void *obj)
 {
 	mqueue_t *q = obj;
 
-	(void) unused_cq;
 	g_assert((q->flags & (MQ_FLOWC|MQ_SWIFT)) == MQ_FLOWC);
 
 	q->flags |= MQ_SWIFT;
 
+	cq_zero(cq, &q->swift_ev);
 	node_tx_swift_changed(q->node);
 	mq_swift_checkpoint(q, TRUE);
 }
@@ -705,7 +669,7 @@ mq_clear(mqueue_t *q)
 	q->flags |= MQ_CLEAR;
 
 	while (q->qhead) {
-		GList *l = q->qhead;
+		plist_t *l = q->qhead;
 		pmsg_t *mb = l->data;
 
 		/*
@@ -775,17 +739,18 @@ mq_flush(mqueue_t *q)
 /**
  * Compare two pointers to links based on their relative priorities, then
  * based on their held Gnutella messages.
- * -- qsort() callback
+ * -- qsort() callback but with an extra data parameter
  */
 static int
-qlink_cmp(const void *a, const void *b)
+qlink_cmp(const void *a, const void *b, void *data)
 {
-	const GList * const *l1 = a, * const *l2 = b;
+	const plist_t * const *l1 = a, * const *l2 = b;
 	const pmsg_t *m1 = (*l1)->data, *m2 = (*l2)->data;
 
-	if (pmsg_prio(m1) == pmsg_prio(m2))
-		return gmsg_cmp(pmsg_start(m1), pmsg_start(m2), TRUE);
-	else
+	if (pmsg_prio(m1) == pmsg_prio(m2)) {
+		const mqueue_t *q = data;
+		return q->uops->msg_cmp(pmsg_start(m1), pmsg_start(m2));
+	} else
 		return pmsg_prio(m1) < pmsg_prio(m2) ? -1 : +1;
 }
 
@@ -795,7 +760,7 @@ qlink_cmp(const void *a, const void *b)
 static void
 qlink_create(mqueue_t *q)
 {
-	GList *l;
+	plist_t *l;
 	int n;
 
 	g_assert(q->qlink == NULL);
@@ -805,18 +770,19 @@ qlink_create(mqueue_t *q)
 	/*
 	 * Prepare sorting of queued messages.
 	 *
-	 * What's sorted is queue links, but the comparison factor is the
-	 * gmsg_cmp() routine to compare the Gnutella messages.
+	 * What's sorted is queue links, but the sorting criteria is the
+	 * user-supplied msg_cmp routine to compare the messages, when they
+	 * have the same priority.
 	 */
 
-	for (l = q->qhead, n = 0; l && n < q->count; l = g_list_next(l), n++) {
+	for (l = q->qhead, n = 0; l && n < q->count; l = plist_next(l), n++) {
 		g_assert(l->data != NULL);
 		q->qlink[n] = l;
 	}
 
 	if (l || n != q->count)
-		g_error("BUG: queue count of %d for %p is wrong (has %d)",
-			q->count, (void *) q, g_list_length(q->qhead));
+		g_error("BUG: queue count of %d for %p is wrong (has %zd)",
+			q->count, (void *) q, plist_length(q->qhead));
 
 	/*
 	 * We use `n' and not `q->count' in case the warning above is emitted,
@@ -824,7 +790,7 @@ qlink_create(mqueue_t *q)
 	 */
 
 	q->qlink_count = n;
-	vsort(q->qlink, n, sizeof q->qlink[0], qlink_cmp);
+	xsort_with_data(q->qlink, n, sizeof q->qlink[0], qlink_cmp, q);
 
 	mq_check(q, 0);
 }
@@ -846,10 +812,10 @@ qlink_free(mqueue_t *q)
  * before the position indicated by `hint'.
  */
 static void
-qlink_insert_before(mqueue_t *q, int hint, GList *l)
+qlink_insert_before(mqueue_t *q, int hint, plist_t *l)
 {
 	g_assert(hint >= 0 && hint < q->qlink_count);
-	g_assert(qlink_cmp(&q->qlink[hint], &l) >= 0);	/* `hint' >= `l' */
+	g_assert(qlink_cmp(&q->qlink[hint], &l, q) >= 0);	/* `hint' >= `l' */
 	g_assert(l->data != NULL);
 
 	mq_check(q, -1);
@@ -870,23 +836,18 @@ qlink_insert_before(mqueue_t *q, int hint, GList *l)
 	q->qlink_count++;
 	HREALLOC_ARRAY(q->qlink, q->qlink_count);
 
-	
-	/* Shift right */
-	memmove(&q->qlink[hint + 1], &q->qlink[hint],
-		(q->qlink_count - hint - 1) * sizeof q->qlink[0]);
-
-	q->qlink[hint] = l;
+	ARRAY_INSERT(q->qlink, hint, q->qlink_count, l);
 }
 
 /**
  * Insert linkable `l' within the sorted qlink array of linkables.
  */
 static void
-qlink_insert(mqueue_t *q, GList *l)
+qlink_insert(mqueue_t *q, plist_t *l)
 {
 	int low = 0;
 	int high = q->qlink_count - 1;
-	GList **qlink = q->qlink;
+	plist_t **qlink = q->qlink;
 
 	g_assert(l->data != NULL);
 
@@ -908,7 +869,7 @@ qlink_insert(mqueue_t *q, GList *l)
 	 * If lower than the beginning, insert at the head.
 	 */
 
-	if (qlink[low] != NULL && qlink_cmp(&l, &qlink[low]) <= 0) {
+	if (qlink[low] != NULL && qlink_cmp(&l, &qlink[low], q) <= 0) {
 		qlink_insert_before(q, low, l);
 		return;
 	}
@@ -917,7 +878,7 @@ qlink_insert(mqueue_t *q, GList *l)
 	 * If higher than the tail, insert at the tail.
 	 */
 
-	if (qlink[high] != NULL && qlink_cmp(&l, &qlink[high]) >= 0) {
+	if (qlink[high] != NULL && qlink_cmp(&l, &qlink[high], q) >= 0) {
 		q->qlink_count++;
 		HREALLOC_ARRAY(q->qlink, q->qlink_count);
 		q->qlink[q->qlink_count - 1] = l;
@@ -993,12 +954,12 @@ qlink_insert(mqueue_t *q, GList *l)
 				return;
 			}
 
-			if (qlink_cmp(&l, &qlink[lowest_non_null]) < 0) {
+			if (qlink_cmp(&l, &qlink[lowest_non_null], q) < 0) {
 				high = lowest_non_null - 1;
 				continue;
 			}
 
-			if (qlink_cmp(&l, &qlink[highest_non_null]) > 0) {
+			if (qlink_cmp(&l, &qlink[highest_non_null], q) > 0) {
 				low = highest_non_null + 1;
 				continue;
 			}
@@ -1017,7 +978,7 @@ qlink_insert(mqueue_t *q, GList *l)
 		 * Regular dichotomic case.
 		 */
 
-		c = qlink_cmp(&qlink[mid], &l);
+		c = qlink_cmp(&qlink[mid], &l, q);
 
 		if (c == 0) {
 			qlink_insert_before(q, mid, l);
@@ -1044,9 +1005,9 @@ qlink_insert(mqueue_t *q, GList *l)
  * @param l			the linkable to remove from the qlink indexer
  */
 static void
-qlink_remove(mqueue_t *q, GList *l)
+qlink_remove(mqueue_t *q, plist_t *l)
 {
-	GList **qlink = q->qlink;
+	plist_t **qlink = q->qlink;
 	int n = q->qlink_count;
 
 	g_assert(qlink);
@@ -1061,12 +1022,12 @@ qlink_remove(mqueue_t *q, GList *l)
 	 */
 
 	if (n > q->count * 3) {
-		GList **dest = qlink;
+		plist_t **dest = qlink;
 		int copied = 0;
 		bool found = FALSE;
 
 		while (n-- > 0) {
-			GList *entry = *qlink++;
+			plist_t *entry = *qlink++;
 			if (entry == NULL)
 				continue;
 			else if (l == entry) {
@@ -1094,9 +1055,9 @@ qlink_remove(mqueue_t *q, GList *l)
 	}
 
 	g_error("BUG: linkable %p for %s not found "
-		"(qlink has %d slots, queue has %d counted items, really %d) at %s:%d",
+		"(qlink has %d slots, queue has %d counted items, really %zd) at %s:%d",
 		(void *) l, mq_info(q),
-		q->qlink_count, q->count, g_list_length(q->qhead),
+		q->qlink_count, q->count, plist_length(q->qhead),
 		_WHERE_, __LINE__);
 }
 
@@ -1120,11 +1081,10 @@ qlink_remove(mqueue_t *q, GList *l)
  */
 static bool
 make_room_internal(mqueue_t *q,
-	char *header, size_t msglen, uint prio, int needed, int *offset)
+	const char *header, size_t msglen, uint prio, int needed, int *offset)
 {
 	int n;
 	int dropped = 0;				/* Amount of messages dropped */
-	bool qlink_corrupted = FALSE;	/* BUG catcher */
 
 	g_assert(needed > 0);
 	mq_check(q, 0);
@@ -1152,7 +1112,7 @@ make_room_internal(mqueue_t *q,
 	 * in the loop below).  When we break out because we found a more
 	 * prioritary message, we remember the index in the array and return it
 	 * to the caller.  If the message is finally inserted in the queue,
-	 * we can insert its GList link right before that index.
+	 * we can insert its plist_t link right before that index.
 	 *
 	 * This adds some complexity but it will avoid millions of calls to
 	 * qlink_cmp(), which is costly.
@@ -1162,9 +1122,8 @@ make_room_internal(mqueue_t *q,
 	 * network, so we can NULLify the corresponding slot in the qlink array.
 	 */
 
-restart:
 	for (n = 0; needed >= 0 && n < q->qlink_count; n++) {
-		GList *item = q->qlink[n];
+		plist_t *item = q->qlink[n];
 		pmsg_t *cmb;
 		char *cmb_start;
 		int cmb_size;
@@ -1176,32 +1135,6 @@ restart:
 		if (item == NULL)
 			continue;
 
-		/*
-		 * BUG catcher -- I've seen situations where item->data is NULL
-		 * at this point, which means something is deeply corrupted...
-		 *		--RAM, 2006-07-14
-		 */
-
-		if (item->data == NULL) {
-			g_error("BUG: NULL data for qlink item #%d/%d in %s at %s:%d",
-				n, q->qlink_count, mq_info(q), _WHERE_, __LINE__);
-
-			if (qlink_corrupted) {
-				g_error(
-					"BUG: trying to ignore still invalid qlink entry at %s:%d",
-						_WHERE_, __LINE__);
-				continue;
-			}
-
-			qlink_corrupted = TRUE;		/* Try to mend it */
-			qlink_free(q);
-			qlink_create(q);
-
-			g_error("BUG: recreated qlink and restarting at %s:%d",
-				_WHERE_, __LINE__);
-			goto restart;
-		}
-
 		cmb = item->data;
 		cmb_start = pmsg_start(cmb);
 
@@ -1222,10 +1155,18 @@ restart:
 		 * (it's necessarily >= 0 if we're in the loop)
 		 */
 
-		if (gmsg_cmp(cmb_start, header, msglen != 0) >= 0) {
-			if (offset != NULL)
-				*offset = n;
-			break;
+		if (0 == msglen) {
+			if (q->uops->msg_headcmp(cmb_start, header) >= 0) {
+				if (offset != NULL)
+					*offset = n;
+				break;
+			}
+		} else {
+			if (q->uops->msg_cmp(cmb_start, header) >= 0) {
+				if (offset != NULL)
+					*offset = n;
+				break;
+			}
 		}
 
 		/*
@@ -1244,15 +1185,17 @@ restart:
 		 * Drop message.
 		 */
 
-		if (MQ_DEBUG_LVL(q) > 4) {
-			gmsg_log_dropped_pmsg(cmb, "to %s %s node %s, in favor of %s",
+		if (MQ_DEBUG_LVL(q) > 4 && q->uops->msg_log != NULL) {
+			q->uops->msg_log(cmb, "to %s %s node %s, in favor of %s",
 				(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
 				NODE_USES_UDP(q->node) ? "UDP" : "TCP",
 				node_addr(q->node), msglen ?
 					gmsg_infostr_full(header, msglen) : gmsg_infostr(header));
 		}
 
-		gnet_stats_count_flowc(pmsg_start(cmb), FALSE);
+		if (q->uops->msg_flowc != NULL)
+			q->uops->msg_flowc(q->node, cmb);
+
 		cmb_size = pmsg_size(cmb);
 
 		g_assert(q->qlink[n] == item);
@@ -1313,9 +1256,9 @@ restart:
  * @returns TRUE if we were able to make enough room.
  */
 static bool
-make_room(mqueue_t *q, pmsg_t *mb, int needed, int *offset)
+make_room(mqueue_t *q, const pmsg_t *mb, int needed, int *offset)
 {
-	char *header = pmsg_start(mb);
+	const char *header = pmsg_start(mb);
 	uint prio = pmsg_prio(mb);
 	size_t msglen = pmsg_written_size(mb);
 
@@ -1328,7 +1271,7 @@ make_room(mqueue_t *q, pmsg_t *mb, int needed, int *offset)
  */
 static bool
 make_room_header(
-	mqueue_t *q, char *header, uint prio, int needed, int *offset)
+	mqueue_t *q, const char *header, uint prio, int needed, int *offset)
 {
 	return make_room_internal(q, header, 0, prio, needed, offset);
 }
@@ -1341,7 +1284,7 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, int msize)
 {
 	int needed;
 	int qlink_offset = -1;
-	GList *new = NULL;
+	plist_t *new = NULL;
 	bool make_room_called = FALSE;
 	bool has_normal_prio = (pmsg_prio(mb) == PMSG_P_DATA);
 
@@ -1361,13 +1304,15 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, int msize)
 		!make_room(q, mb, msize, &qlink_offset)
 	) {
 		g_assert(pmsg_is_unread(mb));			/* Not partially written */
-		if (MQ_DEBUG_LVL(q) > 4)
-			gmsg_log_dropped_pmsg(mb, "to %s %s node %s, %d bytes queued",
+		if (MQ_DEBUG_LVL(q) > 4 && q->uops->msg_log != NULL)
+			q->uops->msg_log(mb, "to %s %s node %s, %d bytes queued",
 				(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
 				NODE_USES_UDP(q->node) ? "UDP" : "TCP",
 				node_addr(q->node), q->size);
 
-		gnet_stats_count_flowc(pmsg_start(mb), FALSE);
+		if (q->uops->msg_flowc != NULL)
+			q->uops->msg_flowc(q->node, mb);
+
 		pmsg_free(mb);
 		node_inc_txdrop(q->node);		/* Dropped during TX */
 		return;
@@ -1397,24 +1342,26 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, int msize)
 
 		g_assert(pmsg_is_unread(mb));			/* Not partially written */
 
-		gnet_stats_count_flowc(pmsg_start(mb), FALSE);
+		if (q->uops->msg_flowc != NULL)
+			q->uops->msg_flowc(q->node, mb);
 
 		if (has_normal_prio) {
-			if (MQ_DEBUG_LVL(q) > 4)
-				gmsg_log_dropped_pmsg(mb,
+			if (MQ_DEBUG_LVL(q) > 4 && q->uops->msg_log != NULL) {
+				q->uops->msg_log(mb,
 					"to %s %s node %s, %d bytes queued [FULL]",
 					(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
 					NODE_USES_UDP(q->node) ? "UDP" : "TCP",
 					node_addr(q->node), q->size);
-
+			}
 			node_inc_txdrop(q->node);		/* Dropped during TX */
 		} else {
-			if (MQ_DEBUG_LVL(q) > 0)
-				gmsg_log_dropped_pmsg(mb,
+			if (MQ_DEBUG_LVL(q) > 0 && q->uops->msg_log != NULL) {
+				q->uops->msg_log(mb,
 					"to %s %s node %s, %d bytes queued [KILLING]",
 					(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
 					NODE_USES_UDP(q->node) ? "UDP" : "TCP",
 					node_addr(q->node), q->size);
+			}
 
 			/*
 			 * Is the check for UDP the correct fix or just a
@@ -1455,15 +1402,15 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, int msize)
 	 */
 
 	if (has_normal_prio) {
-		new = q->qhead = g_list_prepend(q->qhead, mb);
+		new = q->qhead = plist_prepend(q->qhead, mb);
 		if (q->qtail == NULL)
 			q->qtail = q->qhead;
 	} else {
-		GList *l;
+		plist_t *l;
 		uint prio = pmsg_prio(mb);
 		bool inserted = FALSE;
 
-		for (l = q->qtail; l; l = g_list_previous(l)) {
+		for (l = q->qtail; l; l = plist_prev(l)) {
 			pmsg_t *m = l->data;
 
 			if (
@@ -1475,8 +1422,8 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, int msize)
 				 * we are, then leave the loop.
 				 */
 
-				q->qhead = gm_list_insert_after(q->qhead, l, mb);
-				new = g_list_next(l);
+				q->qhead = plist_insert_after(q->qhead, l, mb);
+				new = plist_next(l);
 
 				if (l == q->qtail)				/* Inserted at tail */
 					q->qtail = new;				/* New tail */
@@ -1494,7 +1441,7 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, int msize)
 		if (!inserted) {
 			g_assert(l == NULL);
 
-			new = q->qhead = g_list_prepend(q->qhead, mb);
+			new = q->qhead = plist_prepend(q->qhead, mb);
 			if (q->qtail == NULL)
 				q->qtail = q->qhead;
 		}
