@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002-2003, Raphael Manfredi
+ * Copyright (c) 2002-2003, 2014 Raphael Manfredi
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -28,7 +28,7 @@
  * Message queues.
  *
  * @author Raphael Manfredi
- * @date 2002-2003
+ * @date 2002-2003, 2014
  */
 
 #ifndef _core_mq_h_
@@ -42,11 +42,72 @@
 #include "if/core/mq.h"
 
 #include "lib/cq.h"
+#include "lib/plist.h"
 #include "lib/pmsg.h"
 #include "lib/slist.h"
 
 typedef struct mqueue mqueue_t;
 
+struct mq_ops;
+
+/**
+ * When invoked from the message queue, this callback must return a vector
+ * of "message headers" that are going to be compared against when the queue
+ * is in "swift mode" to determine which messages to prune.
+ *
+ * The messages need not be full message, but must be long enough to allow
+ * the ``msg_headcmp'' callback to compare two messages against each other.
+ *
+ * These messages being templates, they are expected to be generated once only
+ * and then the same vector can be returned.  This is why there is no provision
+ * for a free routine.
+ *
+ * @param initial		whether queue is just entering swift mode
+ * @param vcnt			where the amount of entries in the vector is written
+ *
+ * @return the base of the vector of messages, NULL if none.
+ */
+typedef iovec_t *(*mq_msgtmp_t)(bool initial, size_t *vcnt);
+
+/**
+ * Traffic accounting callback, invoked each time a message has been sent
+ * by the message queue or flow-controlled.
+ *
+ * @param node			the node to which message is sent
+ * @param mb			the full message being accounted for
+ */
+typedef void (*mq_msgcount_t)(void *node, const pmsg_t *mb);
+
+/**
+ * Logging regarding a particular message.
+ *
+ * @param mb			the message being logged
+ * @param fmt			formatting string, followed by arguments to format
+ */
+typedef void (*mq_msglog_t)(const pmsg_t *mb, const char *fmt, ...)
+	G_GNUC_PRINTF(2, 3);
+
+/**
+ * User-supplied parameters, which are callbacks necessary for the message
+ * queue operations but which are dependent on the messages being enqueued.
+ */
+struct mq_uops {
+	cmp_fn_t msg_cmp;			/**< Message (priority) comparison routine */
+	cmp_fn_t msg_headcmp;		/**< Only compare message "headers" */
+	mq_msgtmp_t msg_templates;	/**< Get message templates for "swift" mode */
+	mq_msgcount_t msg_sent;		/**< Message sent */
+	mq_msgcount_t msg_flowc;	/**< Message dropped by flow-control */
+	mq_msgcount_t msg_queued;	/**< Message queued */
+	mq_msglog_t msg_log;		/**< Message logging for dropped messages */
+};
+
+#ifdef MQ_INTERNAL
+
+/*
+ * The declarations in this section are only visible to the various files
+ * implementing the message queue.
+ */
+
 /**
  * Operations defined on all mq types.
  */
@@ -58,18 +119,11 @@ struct mq_ops {
 
 struct mq_cops {
 	void (*puthere)(mqueue_t *q, pmsg_t *mb, int msize);
-	void (*qlink_remove)(mqueue_t *q, GList *l);
-	GList *(*rmlink_prev)(mqueue_t *q, GList *l, int size);
+	void (*qlink_remove)(mqueue_t *q, plist_t *l);
+	plist_t *(*rmlink_prev)(mqueue_t *q, plist_t *l, int size);
 	void (*update_flowc)(mqueue_t *q);
 };
 
-#ifdef MQ_INTERNAL
-
-/*
- * The declarations in this section are only visible to the various files
- * implementing the message queue.
- */
-
 enum mq_magic {
 	MQ_MAGIC = 0x33990ee
 };
@@ -78,7 +132,7 @@ enum mq_magic {
  * A message queue.
  *
  * The queue itself is a two-way list, whose head is kept in `qhead' and the
- * tail in `qtail', since a GList does not keep that information and we
+ * tail in `qtail', since a plist_t does not keep that information and we
  * don't want to traverse the list each time.  Manual bookkeeping required.
  *
  * Flow control is triggered when the size reaches the high watermark,
@@ -94,12 +148,12 @@ enum mq_magic {
  */
 struct mqueue {
 	enum mq_magic magic;	/**< Magic number */
-	gnutella_header_t header;	/**< Comparison point during flow control */
 	struct gnutella_node *node;		/**< Node to which this queue belongs */
 	const struct mq_ops *ops;		/**< Polymorphic operations */
 	const struct mq_cops *cops;		/**< Common operations */
+	const struct mq_uops *uops;		/**< User-defined operations */
 	txdrv_t *tx_drv;				/**< Network TX stack driver */
-	GList *qhead, *qtail, **qlink;
+	plist_t *qhead, *qtail, **qlink;
 	slist_t *qwait;			/**< Waiting queue during putq recursions */
 	cevent_t *swift_ev;		/**< Callout queue event in "swift" mode */
 	const uint32 *debug;	/**< Debug config variable for this queue */
