@@ -40,10 +40,11 @@
 #include "lib/pmsg.h"
 
 struct txdriver;
+struct gnutella_node;
 
 void mq_tcp_putq(mqueue_t *q, pmsg_t *mb, const struct gnutella_node *from);
 mqueue_t *mq_tcp_make(int maxsize,
-	struct gnutella_node *n, struct txdriver *nd);
+	struct gnutella_node *n, struct txdriver *nd, const struct mq_uops *uops);
 
 #endif	/* _core_mq_tcp_h_ */
 
