@@ -44,9 +44,10 @@
  */
 
 struct txdriver;
+struct gnutella_node;
 
-mqueue_t *mq_udp_make(
-	int maxsize, struct gnutella_node *n, struct txdriver *nd);
+mqueue_t *mq_udp_make(int maxsize,
+	struct gnutella_node *n, struct txdriver *nd, const struct mq_uops *uops);
 
 void mq_udp_putq(mqueue_t *q, pmsg_t *mb, const gnet_host_t *to);
 void mq_udp_node_putq(mqueue_t *q, pmsg_t *mb, const struct gnutella_node *n);
