@@ -47,11 +47,11 @@ struct guid;
 void oob_proxy_init(void);
 void oob_proxy_close(void);
 
-gboolean oob_proxy_create(struct gnutella_node *n);
-gboolean oob_proxy_pending_results(
+bool oob_proxy_create(struct gnutella_node *n);
+bool oob_proxy_pending_results(
 	struct gnutella_node *n, const struct guid *muid,
-	int hits, gboolean udp_firewalled, const struct array *token);
-gboolean oob_proxy_got_results(struct gnutella_node *n, guint results);
+	int hits, bool udp_firewalled, const struct array *token);
+bool oob_proxy_got_results(struct gnutella_node *n, uint results);
 const struct guid *oob_proxy_muid_proxied(const struct guid *muid);
 
 #endif /* _core_oob_proxy_h_ */
