@@ -366,7 +366,7 @@ route_udp_node_hash(const void *key)
 {
 	const struct routing_udp_node *un = key;
 
-	return host_addr_hash(un->addr) ^ port_hash(un->port);
+	return host_addr_port_hash(un->addr, un->port);
 }
 
 /**
@@ -857,6 +857,37 @@ message_check(const struct message * const m, unsigned chunk_idx)
 	g_assert(chunk_idx == m->chunk_idx);
 }
 
+/**
+ * Asserts that a message entry is consistent and belongs to the correct chunk.
+ *
+ * The "chunk" parameter points to the old chunk address and is used to
+ * properly validate the m->slot value (before our caller can update it
+ * to reflect the new chunk location).
+ */
+static void
+message_check_chunk(const struct message * const m, unsigned chunk_idx,
+	struct message * const *chunk)
+{
+	const void *chunk_end;
+
+	g_assert(m != NULL);
+	g_assert(chunk_idx == m->chunk_idx);
+
+	/*
+	 * Expand a variant of slot_check() since routing.chunks[chunk_idx]
+	 * has been re-allocated and the old chunk address is given.
+	 */
+
+	g_assert(uint_is_non_negative(chunk_idx));
+	g_assert(chunk_idx < MAX_CHUNKS);
+	g_assert(chunk_idx < routing.nchunks);
+
+	chunk_end = const_ptr_add_offset(chunk, CHUNK_MESSAGES * sizeof m);
+
+	g_assert(ptr_cmp(m->slot, chunk) >= 0);
+	g_assert(ptr_cmp(m->slot, chunk_end) < 0);
+}
+
 /**
  * Clean already allocated entry.
  */
@@ -962,7 +993,7 @@ routing_chunk_move(struct message **chunk, unsigned chunk_idx)
 		struct message *m = *p;
 
 		if (m != NULL) {
-			message_check(m, chunk_idx);
+			message_check_chunk(m, chunk_idx, chunk);
 			m->slot = p;
 		}
 	}
@@ -1412,7 +1443,7 @@ message_hash_func(const void *key)
 {
 	const struct message *msg = key;
 
-	return integer_hash(msg->function) ^
+	return integer_hash_fast(msg->function) ^
 		universal_hash(&msg->muid, GUID_RAW_SIZE);
 }
 
@@ -2905,6 +2936,8 @@ route_message(gnutella_node_t **node, struct route_dest *dest)
 	const guid_t *muid = NULL;
 	uint8 function;
 
+	node_check(sender);
+
 	function = gnutella_header_get_function(&sender->header);
 	muid = gnutella_header_get_muid(&sender->header);
 
@@ -2948,6 +2981,13 @@ route_message(gnutella_node_t **node, struct route_dest *dest)
 
 		dest->duplicate = duplicate = booleanize(m != NULL);
 
+		/*
+		 * If the node has been removed, we won't handle the message.
+		 */
+
+		if G_UNLIKELY(NULL == *node)
+			goto done;
+
 		/*
 		 * Record the message in the routing table.
 		 *
