@@ -41,6 +41,8 @@
 
 #include "spam.h"
 
+#include "settings.h"
+
 #include "lib/ascii.h"
 #include "lib/atoms.h"
 #include "lib/dbmw.h"
@@ -305,19 +307,9 @@ spam_sha1_retrieve(void)
 	file_path_t fp[4];
 	FILE *f;
 	int idx;
-	char *tmp;
-	unsigned length = 0;
+	unsigned length;
 
-	file_path_set(&fp[length++], settings_config_dir(), spam_sha1_file);
-	file_path_set(&fp[length++], PRIVLIB_EXP, spam_sha1_file);
-
-#ifndef OFFICIAL_BUILD
-	file_path_set(&fp[length++], PACKAGE_EXTRA_SOURCE_DIR, spam_sha1_file);
-#endif	/* !OFFICIAL_BUILD */
-
-	tmp = get_folder_path(PRIVLIB_PATH, NULL);
-	if (tmp != NULL)
-		file_path_set(&fp[length++], tmp, spam_sha1_file);
+	length = settings_file_path_load(fp, spam_sha1_file, SFP_DFLT);
 
 	g_assert(length <= G_N_ELEMENTS(fp));
 
@@ -326,8 +318,6 @@ spam_sha1_retrieve(void)
 		spam_sha1_retrieve_from_file(f, fp[idx].dir, fp[idx].name);
 		fclose(f);
 	}
-
-	HFREE_NULL(tmp);
 }
 
 /**
