@@ -51,6 +51,8 @@
 #include "search.h"
 #include "settings.h"
 
+#include "g2/msg.h"
+
 #include "if/gnet_property_priv.h"
 
 #include "lib/atoms.h"
@@ -88,7 +90,7 @@ struct smsg_info {
 
 static squeue_t *global_sq = NULL;
 
-static void cap_queue(squeue_t *sq);
+static void sq_cap(squeue_t *sq);
 
 /**
  * Free routine for a query message.
@@ -162,7 +164,7 @@ smsg_discard(smsg_t *sb)
  * the mq to which it will be sent to.
  */
 static void
-smsg_mutate(smsg_t *sb, struct gnutella_node *n)
+smsg_mutate(smsg_t *sb, const gnutella_node_t *n)
 {
 	struct smsg_info *smi;
 	pmsg_t *omb;
@@ -176,6 +178,20 @@ smsg_mutate(smsg_t *sb, struct gnutella_node *n)
 	pmsg_free(omb);
 }
 
+/**
+ * Extract query text from enqueued message block (for logging purposes).
+ *
+ * @param sb		the enqueued search message object
+ * @param n			node to which message should be sent (to get the protocol)
+ */
+static const char *
+smsg_query_text(const smsg_t *sb, const gnutella_node_t *n)
+{
+	return (n != NULL && NODE_TALKS_G2(n)) ?
+		g2_msg_search_get_text(sb->mb) :
+		gnutella_msg_search_get_text(pmsg_start(sb->mb));
+}
+
 /***
  *** "handle" hash table management.
  ***/
@@ -230,7 +246,7 @@ sqh_remove(squeue_t *sq, gnet_search_t sh)
  * Create a new search queue.
  */
 squeue_t *
-sq_make(struct gnutella_node *node)
+sq_make(gnutella_node_t *node)
 {
     squeue_t *sq;
 
@@ -262,22 +278,16 @@ sq_make(struct gnutella_node *node)
 void
 sq_clear(squeue_t *sq)
 {
-	GList *l;
-
 	g_assert(sq);
 
 	if (GNET_PROPERTY(sq_debug) > 3)
-		g_debug("clearing sq node %s (sent=%d, dropped=%d)",
-			sq->node ? node_addr(sq->node) : "GLOBAL",
+		g_debug("%s(): clearing sq for %s (sent=%d, dropped=%d)",
+			G_STRFUNC, sq->node ? node_infostr(sq->node) : "GLOBAL",
 			sq->n_sent, sq->n_dropped);
 
-	for (l = sq->searches; l; l = g_list_next(l)) {
-		smsg_t *sb = l->data;
+	PLIST_FOREACH_CALL(sq->searches, smsg_discard);
 
-		smsg_discard(sb);
-	}
-
-	gm_list_free_null(&sq->searches);
+	plist_free_null(&sq->searches);
 	sq->count = 0;
 }
 
@@ -315,11 +325,11 @@ sq_puthere(squeue_t *sq, gnet_search_t sh, pmsg_t *mb, query_hashvec_t *qhv)
 	sb = smsg_alloc(sh, mb, qhv);
 
 	sqh_put(sq, sh);
-	sq->searches = g_list_prepend(sq->searches, sb);
+	sq->searches = plist_prepend(sq->searches, sb);
 	sq->count++;
 
 	if (sq->count > GNET_PROPERTY(search_queue_size))
-		cap_queue(sq);
+		sq_cap(sq);
 }
 
 
@@ -362,9 +372,9 @@ void
 sq_process(squeue_t *sq, time_t now)
 {
     time_delta_t spacing = GNET_PROPERTY(search_queue_spacing);
-	GList *item;
+	plist_t *item;
 	smsg_t *sb;
-	struct gnutella_node *n;
+	gnutella_node_t *n;
 	bool sent;
 
 	g_assert(sq->node == NULL || sq->node->outq != NULL);
@@ -421,7 +431,7 @@ retry:
 
 	g_assert(sq->searches);
 
-	item = g_list_first(sq->searches);
+	item = plist_first(sq->searches);
 	sb = item->data;
 
 	g_assert(sq->count > 0);
@@ -431,10 +441,13 @@ retry:
 	if (n == NULL) {
 		g_assert(sb->qhv != NULL);		/* Enqueued via sq_global_putq() */
 
-		if (GNET_PROPERTY(sq_debug) > 2)
-			g_debug("sq GLOBAL, queuing \"%s\" (%u left, %d sent)",
-				gnutella_msg_search_get_text(pmsg_start(sb->mb)),
+		/* This is not used by G2 */
+
+		if (GNET_PROPERTY(sq_debug) > 2) {
+			g_debug("%s(): GLOBAL sq, queuing \"%s\" (%u left, %d sent)",
+				G_STRFUNC, gnutella_msg_search_get_text(pmsg_start(sb->mb)),
 				sq->count, sq->n_sent);
+		}
 
 		dq_launch_local(sb->shandle, sb->mb, sb->qhv);
 
@@ -446,10 +459,11 @@ retry:
 
 		g_assert(sb->qhv == NULL);		/* Enqueued via sq_putq() */
 
-		if (GNET_PROPERTY(sq_debug) > 2)
-			g_debug("sq for node %s, queuing \"%s\" (%u left, %d sent)",
-				node_addr(n), gnutella_msg_search_get_text(pmsg_start(sb->mb)),
+		if (GNET_PROPERTY(sq_debug) > 2) {
+			g_debug("%s(): sq for %s, queuing \"%s\" (%u left, %d sent)",
+				G_STRFUNC, node_infostr(n), smsg_query_text(sb, n),
 				sq->count, sq->n_sent);
+		}
 
 		/*
 		 * If we're a leaf node, we're doing a leaf-guided dynamic query.
@@ -458,31 +472,33 @@ retry:
 		 * queries that go out.
 		 */
 
-		if (settings_is_leaf())
+		if (settings_is_leaf() && !NODE_TALKS_G2(n))
 			smsg_mutate(sb, n);
 
 		mq_tcp_putq(n->outq, sb->mb, NULL);
 
 	} else {
-		if (GNET_PROPERTY(sq_debug) > 4)
-			g_debug("sq for node %s, ignored \"%s\" (%u left, %d sent)",
-				node_addr(n), gnutella_msg_search_get_text(pmsg_start(sb->mb)),
+		if (GNET_PROPERTY(sq_debug) > 4) {
+			g_debug("%s(): sq for %s, ignored \"%s\" (%u left, %d sent)",
+				G_STRFUNC, node_infostr(n), smsg_query_text(sb, n),
 				sq->count, sq->n_sent);
-		pmsg_free(sb->mb);
-		if (sb->qhv)
-			qhvec_free(sb->qhv);
+		}
+
 		sent = FALSE;
 	}
 
+	sqh_remove(sq, sb->shandle);
+
 	if (sent) {
 		sq->n_sent++;
 		sq->last_sent = now;
+		smsg_free(sb);
+	} else {
+		smsg_discard(sb);
 	}
 
-	sqh_remove(sq, sb->shandle);
-	smsg_free(sb);
-	sq->searches = g_list_remove_link(sq->searches, item);
-	g_list_free_1(item);
+	sq->searches = plist_remove_link(sq->searches, item);
+	plist_free_1(item);
 
 	/*
 	 * If we ignored the query, retry with the next in the queue.
@@ -498,28 +514,33 @@ retry:
  * search queue based on the search count
  */
 static void
-cap_queue(squeue_t *sq)
+sq_cap(squeue_t *sq)
 {
-    while (sq->count > GNET_PROPERTY(search_queue_size)) {
-    	GList *item = g_list_last(sq->searches);
-		smsg_t *sb = item->data;
+	plist_t *last = NULL;
 
-		sq->searches = g_list_remove_link(sq->searches, item);
+	while (sq->count > GNET_PROPERTY(search_queue_size)) {
+		plist_t *item;
+		smsg_t *sb;
+
+		item = NULL == last ? plist_last(sq->searches) : last;
+		sb = item->data;
+		last = plist_prev(item);
+		sq->searches = plist_remove_link(sq->searches, item);
+		plist_free_1(item);
 
 		g_assert(sq->count > 0);
 		sq->count--;
 		sq->n_dropped++;
 
-		if (GNET_PROPERTY(sq_debug) > 4)
-			g_debug("sq for node %s, dropped \"%s\" (%u left, %d dropped)",
-				node_addr(sq->node),
-				gnutella_msg_search_get_text(pmsg_start(sb->mb)),
-				sq->count, sq->n_dropped);
+		if (GNET_PROPERTY(sq_debug) > 4) {
+			g_debug("%s(): sq for %s, dropped \"%s\" (%u left, %d dropped)",
+				G_STRFUNC, node_infostr(sq->node),
+				smsg_query_text(sb, sq->node), sq->count, sq->n_dropped);
+		}
 
 		sqh_remove(sq, sb->shandle);
 		smsg_discard(sb);
-		g_list_free_1(item);
-    }
+	}
 }
 
 /**
@@ -529,29 +550,29 @@ cap_queue(squeue_t *sq)
 void
 sq_search_closed(squeue_t *sq, gnet_search_t sh)
 {
-	GList *l;
-	GList *next;
+	plist_t *l, *next;
 
 	for (l = sq->searches; l; l = next) {
 		smsg_t *sb = l->data;
 
-		next = g_list_next(l);
+		next = plist_next(l);
 
 		if (sb->shandle != sh)
 			continue;
 
 		g_assert(sq->count > 0);
 		sq->count--;
-		sq->searches = g_list_remove_link(sq->searches, l);
+		sq->searches = plist_remove_link(sq->searches, l);
 
-		if (GNET_PROPERTY(sq_debug) > 4)
-			g_debug("sq for node %s, dropped \"%s\" on search close (%u left)",
-				sq->node ? node_addr(sq->node) : "GLOBAL",
-				gnutella_msg_search_get_text(pmsg_start(sb->mb)), sq->count);
+		if (GNET_PROPERTY(sq_debug) > 4) {
+			g_debug("%s(): sq for %s, dropped \"%s\" on search close (%u left)",
+				G_STRFUNC, sq->node ? node_infostr(sq->node) : "GLOBAL",
+				smsg_query_text(sb, sq->node), sq->count);
+		}
 
 		sqh_remove(sq, sb->shandle);
 		smsg_discard(sb);
-		g_list_free_1(l);
+		plist_free_1(l);
 	}
 
 	g_assert(sq->searches || sq->count == 0);
