@@ -385,7 +385,7 @@ udp_sched_write_error(const udp_sched_t *us, const gnet_host_t *to,
 		/*
 		 * We don't care about lost packets.
 		 */
-		g_carp("%s(): UDP write of %d bytes to %s failed: %m",
+		g_warning("%s(): UDP write of %d bytes to %s failed: %m",
 			func, pmsg_size(mb), gnet_host_to_string(to));
 		break;
 	default:
