@@ -64,7 +64,7 @@ struct verify_hash {
 struct verify *verify_new(const struct verify_hash *);
 void verify_free(struct verify **ptr);
 
-int verify_enqueue(struct verify *, int high_priority,
+bool verify_enqueue(struct verify *, int high_priority,
 	const char *pathname, filesize_t offset, filesize_t filesize,
 	verify_callback callback, void *user_data);
 
