@@ -154,7 +154,9 @@
 #include "lib/htable.h"
 #include "lib/nid.h"
 #include "lib/patricia.h"
+#include "lib/pslist.h"
 #include "lib/slist.h"
+#include "lib/stringify.h"
 #include "lib/tm.h"
 #include "lib/unsigned.h"
 #include "lib/walloc.h"
@@ -432,7 +434,7 @@ publish_final_stats(publish_t *pb)
 		g_debug("DHT PUBLISH[%s] %g secs published %d/%d (%d error%s) "
 			"in=%d bytes, out=%d bytes",
 			nid_to_string(&pb->pid), tm_elapsed_f(&end, &pb->start),
-			pb->published, pb->cnt, pb->errors, 1 == pb->errors ? "" : "s",
+			pb->published, pb->cnt, pb->errors, plural(pb->errors),
 			pb->bw_incoming, pb->bw_outgoing);
 }
 
@@ -495,7 +497,7 @@ path_loaded:
 	if (GNET_PROPERTY(dht_publish_debug) > 1) {
 		size_t count = patricia_count(path);
 		g_debug("DHT PUBLISH[%s] updating roots cache with %zu entr%s near %s",
-			nid_to_string(&pb->pid), count, 1 == count ? "y" : "ies",
+			nid_to_string(&pb->pid), count, plural_y(count),
 			kuid_to_hex_string(pb->key));
 	}
 
@@ -521,8 +523,7 @@ publish_terminate(publish_t *pb, publish_error_t code)
 			(pb->flags & PB_F_BACKGROUND) ? "background " : "",
 			PUBLISH_VALUE == pb->type ? "to" : "of",
 			pb->published, pb->cnt,
-			PUBLISH_VALUE == pb->type ? "root" : "item",
-			1 == pb->cnt ? "" : "s",
+			PUBLISH_VALUE == pb->type ? "root" : "item", plural(pb->cnt),
 			pb->key ? kuid_to_hex_string(pb->key) : "<no key>",
 			publish_strerror(code));
 	}
@@ -637,7 +638,7 @@ publish_cancel(publish_t *pb, bool callback)
 			(pb->flags & PB_F_SUBORDINATE) ? "subordinate " : "",
 			(pb->flags & PB_F_BACKGROUND) ? "background " : "",
 			publish_type_to_string(pb->type),
-			pb->published, pb->cnt, 1 == pb->cnt ? "" : "s",
+			pb->published, pb->cnt, plural(pb->cnt),
 			kuid_to_hex_string(pb->key));
 	}
 
@@ -664,14 +665,13 @@ publish_cancel(publish_t *pb, bool callback)
  * Expiration timeout on caching requesst.
  */
 static void
-publish_cache_expired(cqueue_t *unused_cq, void *obj)
+publish_cache_expired(cqueue_t *cq, void *obj)
 {
 	publish_t *pb = obj;
 
-	(void) unused_cq;
 	publish_check(pb);
 
-	pb->expire_ev = NULL;
+	cq_zero(cq, &pb->expire_ev);
 
 	if (GNET_PROPERTY(dht_publish_debug))
 		g_debug("DHT PUBLISH[%s] %s%s%s publish of %d value%s for %s expired",
@@ -679,7 +679,7 @@ publish_cache_expired(cqueue_t *unused_cq, void *obj)
 			(pb->flags & PB_F_SUBORDINATE) ? "subordinate " : "",
 			(pb->flags & PB_F_BACKGROUND) ? "background " : "",
 			publish_type_to_string(pb->type),
-			pb->cnt, 1 == pb->cnt ? "" : "s",
+			pb->cnt, plural(pb->cnt),
 			kuid_to_hex_string(pb->key));
 
 	publish_terminate(pb, PUBLISH_E_EXPIRED);
@@ -689,19 +689,18 @@ publish_cache_expired(cqueue_t *unused_cq, void *obj)
  * Expiration timeout for offloading.
  */
 static void
-publish_offload_expired(cqueue_t *unused_cq, void *obj)
+publish_offload_expired(cqueue_t *cq, void *obj)
 {
 	publish_t *pb = obj;
 
-	(void) unused_cq;
 	publish_check(pb);
 
-	pb->expire_ev = NULL;
+	cq_zero(cq, &pb->expire_ev);
 
 	if (GNET_PROPERTY(dht_publish_debug))
 		g_debug("DHT PUBLISH[%s] %s publish of %d key%s to %s expired",
 			nid_to_string(&pb->pid), publish_type_to_string(pb->type),
-			pb->cnt, 1 == pb->cnt ? "" : "s",
+			pb->cnt, plural(pb->cnt),
 			knode_to_string(pb->target.o.kn));
 
 	publish_terminate(pb, PUBLISH_E_EXPIRED);
@@ -711,21 +710,20 @@ publish_offload_expired(cqueue_t *unused_cq, void *obj)
  * Expiration timeout for STORE.
  */
 static void
-publish_store_expired(cqueue_t *unused_cq, void *obj)
+publish_store_expired(cqueue_t *cq, void *obj)
 {
 	publish_t *pb = obj;
 
-	(void) unused_cq;
 	publish_check(pb);
 
-	pb->expire_ev = NULL;
+	cq_zero(cq, &pb->expire_ev);
 
 	if (GNET_PROPERTY(dht_publish_debug))
 		g_debug("DHT PUBLISH[%s] %s publish of %s expired: "
 			"published to %d/%d root%s",
 			nid_to_string(&pb->pid), publish_type_to_string(pb->type),
 			dht_value_to_string(pb->target.v.value),
-			pb->published, pb->cnt, 1 == pb->cnt ? "" : "s");
+			pb->published, pb->cnt, plural(pb->cnt));
 
 	publish_terminate(pb, PUBLISH_E_EXPIRED);
 }
@@ -758,9 +756,8 @@ log_status(publish_t *pb)
 		nid_to_string(&pb->pid), 
 		PUBLISH_VALUE == pb->type ? "to " : "",
 		pb->published, pb->cnt,
-		PUBLISH_VALUE == pb->type ? "root" : "item",
-		1 == pb->cnt ? "" : "s",
-		pb->errors, 1 == pb->errors ? "" : "s");
+		PUBLISH_VALUE == pb->type ? "root" : "item", plural(pb->cnt),
+		pb->errors, plural(pb->errors));
 }
 
 /**
@@ -818,18 +815,16 @@ first_creator_kuid(pmsg_t *mb)
  * Delay expiration.
  */
 static void
-publish_delay_expired(cqueue_t *unused_cq, void *obj)
+publish_delay_expired(cqueue_t *cq, void *obj)
 {
 	publish_t *pb = obj;
 
-	(void) unused_cq;
-
 	if (G_UNLIKELY(NULL == publishes))
 		return;		/* Shutdown occurred */
 
 	publish_check(pb);
 
-	pb->delay_ev = NULL;
+	cq_zero(cq, &pb->delay_ev);
 	pb->flags &= ~PB_F_DELAYED;
 	publish_iterate(pb);
 }
@@ -932,7 +927,7 @@ publish_handle_reply(publish_t *pb, const knode_t *kn,
 		g_warning("DHT PUBLISH[%s] STORE ACK from %s has %u status%s "
 			"(expected %u)",
 			nid_to_string(&pb->pid), knode_to_string(kn),
-			acks, 1 == acks ? "" : "es", published);
+			acks, plural_es(acks), published);
 
 		if (acks > published)
 			goto ignore;		/* How can remote send us more acks? */
@@ -1062,8 +1057,8 @@ publish_handle_reply(publish_t *pb, const knode_t *kn,
 		g_warning("DHT PUBLISH[%s] the STORE_RESPONSE payload (%lu byte%s) "
 			"from %s has %lu byte%s of unparsed trailing data (ignored)",
 			 nid_to_string(&pb->pid),
-			 (gulong) len, len == 1 ? "" : "s", knode_to_string(kn),
-			 (gulong) unparsed, 1 == unparsed ? "" : "s");
+			 (ulong) len, plural(len), knode_to_string(kn),
+			 (ulong) unparsed, plural(unparsed));
 	}
 
 	/* FALL THROUGH */
@@ -1102,7 +1097,7 @@ bad:
 		g_warning("DHT PUBLISH[%s] improper STORE_RESPONSE payload "
 			"(%zu byte%s) from %s: %s%s%s",
 			nid_to_string(&pb->pid),
-			len, len == 1 ? "" : "s", knode_to_string(kn), reason,
+			len, plural(len), knode_to_string(kn), reason,
 			bstr_has_error(bs) ? ": " : "",
 			bstr_has_error(bs) ? bstr_error(bs) : "");
 
@@ -1295,7 +1290,7 @@ pb_msg_dropped(void *obj, knode_t *unused_kn, pmsg_t *mb)
 			uint8 held = values_held(mb);
 			const kuid_t *id = first_creator_kuid(mb);
 			g_debug("DHT PUBLISH[%s] UDP dropped STORE with %u value%s sk=%s",
-				nid_to_string(&pb->pid), held, 1 == held ? "" : "s",
+				nid_to_string(&pb->pid), held, plural(held),
 				kuid_to_hex_string(id));
 		}
 	} else {
@@ -1306,7 +1301,7 @@ pb_msg_dropped(void *obj, knode_t *unused_kn, pmsg_t *mb)
 			const kuid_t *id = first_creator_kuid(mb);
 			g_debug("DHT PUBLISH[%s] "
 				"synchronous UDP drop of STORE with %u value%s sk=%s",
-				nid_to_string(&pb->pid), held, 1 == held ? "" : "s",
+				nid_to_string(&pb->pid), held, plural(held),
 				kuid_to_hex_string(id));
 		}
 	}
@@ -1391,7 +1386,7 @@ pb_cache_handling_rpc(void *obj, enum dht_rpc_ret type,
 			if (GNET_PROPERTY(dht_publish_debug) > 1) {
 				uint8 held = values_held(pb->target.c.pending);
 				g_debug("DHT PUBLISH[%s] dropping publishing of %u value%s",
-					nid_to_string(&pb->pid), held, 1 == held ? "" : "s");
+					nid_to_string(&pb->pid), held, plural(held));
 			}
 			pmsg_free(mbp);
 			pb->target.c.timeouts = 0;
@@ -1704,7 +1699,7 @@ publish_cache_send(publish_t *pb, pmsg_t *mb)
 			nid_to_string(&pb->pid), pb->hops, pmsg_size(mb),
 			pb->target.c.timeouts + 1,
 			kuid_to_hex_string(first_creator_kuid(mb)),
-			held, 1 == held ? "" : "s");
+			held, plural(held));
 	}
 
 	revent_store(pb->target.c.kn, mb, pb->pid, &publish_cache_ops, pb->hops);
@@ -1789,9 +1784,8 @@ pb_offload_child_done(void *obj, int count, int published, int errors,
 			"in=%d bytes, out=%d bytes",
 			nid_to_string(&pb->pid),
 			tm_elapsed_f(&now, &pb->start), pb->hops,
-			published, count, 1 == published ? "" : "s",
-			errors, 1 == errors ? "" : "s",
-			bw_incoming, bw_outgoing);
+			published, count, plural(published),
+			errors, plural(errors), bw_incoming, bw_outgoing);
 	}
 
 	pb->target.o.child = NULL;
@@ -1856,7 +1850,7 @@ publish_offload_iterate(publish_t *pb)
 				"%g secs, hop %u: offloaded key %s has %d value%s",
 				nid_to_string(&pb->pid),
 				tm_elapsed_f(&now, &pb->start), pb->hops,
-				kuid_to_hex_string(key), valcnt, 1 == valcnt ? "" : "s");
+				kuid_to_hex_string(key), valcnt, plural(valcnt));
 		}
 
 		if (valcnt > 0) {
@@ -1924,7 +1918,7 @@ static void
 publish_value_iterate(publish_t *pb)
 {
 	pmsg_t *mb;
-	GSList *sl;
+	pslist_t *sl;
 	lookup_rc_t *rc;
 
 	publish_check(pb);
@@ -1970,10 +1964,10 @@ publish_value_iterate(publish_t *pb)
 	sl = kmsg_build_store(rc->token, rc->token_len, &pb->target.v.value, 1);
 
 	g_assert(sl != NULL);
-	g_assert(g_slist_length(sl) == 1);
+	g_assert(pslist_length(sl) == 1);
 
 	mb = sl->data;
-	g_slist_free(sl);
+	pslist_free(sl);
 
 	/*
 	 * Send message to node.
@@ -2071,8 +2065,8 @@ publish_create(const kuid_t *key, publish_type_t type, int cnt)
 			"starting %s publishing %s %d %s%s for %s",
 			nid_to_string(&pb->pid), publish_type_to_string(pb->type),
 			PUBLISH_VALUE == pb->type ? "to" : "of",
-			cnt, PUBLISH_VALUE == pb->type ? "root" : "item",
-			1 == cnt ? "" : "s", kuid_to_hex_string(pb->key));
+			cnt, PUBLISH_VALUE == pb->type ? "root" : "item", plural(cnt),
+			kuid_to_hex_string(pb->key));
 	}
 
 	htable_insert(publishes, &pb->pid, pb);
@@ -2101,8 +2095,8 @@ publish_cache_internal(const kuid_t *key,
 	lookup_rc_t *target, dht_value_t **vvec, int vcnt)
 {
 	publish_t *pb;
-	GSList *msg;
-	GSList *sl;
+	pslist_t *msg;
+	pslist_t *sl;
 	int vheld = 0;
 
 	/* Make sure all values bear the same primary key */
@@ -2127,13 +2121,13 @@ publish_cache_internal(const kuid_t *key,
 
 	msg = kmsg_build_store(target->token, target->token_len, vvec, vcnt);
 
-	GM_SLIST_FOREACH(msg, sl) {
+	PSLIST_FOREACH(msg, sl) {
 		pmsg_t *mb = sl->data;
 
 		slist_append(pb->target.c.messages, mb);
 		vheld += values_held(mb);
 	}
-	g_slist_free(msg);
+	pslist_free(msg);
 
 	g_assert(vheld == vcnt);	/* We have all our values in the messages */
 
@@ -2141,7 +2135,7 @@ publish_cache_internal(const kuid_t *key,
 		g_debug("DHT PUBLISH[%s] to %s (security token: %u byte%s)",
 			nid_to_string(&pb->pid),
 			knode_to_string(target->kn), target->token_len,
-			1 == target->token_len ? "" : "s");
+			plural(target->token_len));
 	}
 
 	if (GNET_PROPERTY(dht_publish_debug) > 3) {
@@ -2246,7 +2240,7 @@ pb_token_found(const kuid_t *kuid, const lookup_rs_t *rs, void *arg)
 			"offloading got security token (%d byte%s) for %s",
 			nid_to_string(&pb->pid),
 			tm_elapsed_f(&now, &pb->start),
-			rc->token_len, 1 == rc->token_len ? "" : "s",
+			rc->token_len, plural(rc->token_len),
 			knode_to_string(pb->target.o.kn));
 	}
 
@@ -2304,11 +2298,11 @@ pb_token_lookup_stats(const kuid_t *kuid,
  * @return created publishing object.
  */
 publish_t *
-publish_offload(const knode_t *kn, GSList *keys)
+publish_offload(const knode_t *kn, pslist_t *keys)
 {
 	publish_t *pb;
 	slist_t *skeys;
-	GSList *sl;
+	pslist_t *sl;
 	uint8 toklen;
 	const void *token;
 
@@ -2317,7 +2311,7 @@ publish_offload(const knode_t *kn, GSList *keys)
 
 	skeys = slist_new();
 
-	GM_SLIST_FOREACH(keys, sl) {
+	PSLIST_FOREACH(keys, sl) {
 		kuid_t *id = sl->data;
 		slist_append(skeys, kuid_get_atom(id));
 	}
