@@ -1,6 +1,4 @@
 /*
- * $Id: downloads.h 10201 2005-11-17 20:09:09Z rmanfredi $
- *
  * Copyright (c) 2004, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -26,7 +24,7 @@
 #ifndef _if_ui_gtk_downloads_h_
 #define _if_ui_gtk_downloads_h_
 
-#include <glib.h>
+#include "common.h"
 
 /*
  * Public interface, visible from the bridge.
@@ -34,23 +32,8 @@
 
 #ifdef GUI_SOURCES
 
-struct download;
-
-void gui_download_enable_start_now(
-	guint32 running_downloads, guint32 max_downloads);
-
-void download_gui_add(struct download *d);
-void download_gui_remove(struct download *d);
-
-void gui_update_download(struct download *, gboolean);
-void gui_update_download_server(struct download *);
-void gui_update_download_range(struct download *d);
-void gui_update_download_size(struct download *d);
-void gui_update_download_host(struct download *d);
-void gui_update_download_abort_resume(void);
-void gui_update_download_clear(void);
-void gui_update_queue_frozen(void);
-void gui_update_download_clear_now(void);
+void gui_download_updates_freeze(void);
+void gui_download_updates_thaw(void);
 
 #endif /* GUI_SOURCES */
 #endif /* _if_ui_gtk_downloads_h_ */
