@@ -60,6 +60,32 @@ typedef bool (*cq_invoke_t)(void *udata);
 
 typedef uint64 cq_time_t;		/**< Virtual time for callout queue */
 
+enum cq_info_magic { CQ_INFO_MAGIC = 0x12c867d4 };
+
+/**
+ * Callout queue information that can be retrieved.
+ */
+typedef struct {
+	enum cq_info_magic magic;
+	const char *name;			/**< Queue name (atom) */
+	const char *parent;			/**< Parent queue name (atom), NULL if none */
+	uint stid;					/**< Scheduling thread ID */
+	size_t periodic_count;		/**< Amount of periodic events */
+	size_t idle_count;			/**< Amount of idle events */
+	size_t event_count;			/**< Amount of registered events */
+	size_t heartbeat_count;		/**< Amount of heartbeats */
+	size_t triggered_count;		/**< Amount of triggered events */
+	int period;					/**< Period, in ms */
+	time_t last_idle;			/**< Last idle scheduling */
+} cq_info_t;
+
+static inline void
+cq_info_check(const cq_info_t * const cqi)
+{
+	g_assert(cqi != NULL);
+	g_assert(CQ_INFO_MAGIC == cqi->magic);
+}
+
 /*
  * Interface routines.
  */
@@ -67,7 +93,7 @@ typedef uint64 cq_time_t;		/**< Virtual time for callout queue */
 double cq_main_coverage(int old_ticks);
 
 void cq_init(cq_invoke_t idle, const uint32 *debug);
-void cq_dispatch(void);
+void cq_main_dispatch(void);
 void cq_halt(void);
 void cq_close(void);
 
@@ -79,15 +105,20 @@ void cq_free_null(cqueue_t **cq_ptr);
 cevent_t *cq_insert(cqueue_t *cq, int delay, cq_service_t fn, void *arg);
 cevent_t *cq_main_insert(int delay, cq_service_t fn, void *arg);
 cq_time_t cq_remaining(const cevent_t *ev);
-void cq_expire(cevent_t *ev);
-void cq_cancel(cevent_t **handle_ptr);
-void cq_replace(cevent_t *ev, cq_service_t fn, void *arg);
-void cq_resched(cevent_t *handle, int delay);
+size_t cq_heartbeat(cqueue_t *cq);
+bool cq_expire(cevent_t *ev);
+void cq_zero(cqueue_t *cq, cevent_t **ev_ptr);
+bool cq_zero_if_triggered(cevent_t **ev_ptr);
+bool cq_cancel(cevent_t **handle_ptr);
+bool cq_replace(cevent_t *ev, cq_service_t fn, void *arg);
+bool cq_resched(cevent_t *handle, int delay);
 int cq_ticks(const cqueue_t *cq);
 int cq_count(const cqueue_t *cq);
+int cq_delay(const cqueue_t *cq);
 const char *cq_name(const cqueue_t *cq);
-void cq_idle(cqueue_t *cq);
-void cq_main_idle(void);
+size_t cq_idle(cqueue_t *cq);
+size_t cq_main_idle(void);
+unsigned cq_main_thread_id(void);
 
 cperiodic_t *cq_periodic_add(cqueue_t *cq,
 	int period, cq_invoke_t event, void *arg);
@@ -99,6 +130,9 @@ void cq_idle_remove(cidle_t **ci_ptr);
 
 const char *cq_time_to_string(cq_time_t t);
 
+struct pslist *cq_info_list(void);
+void cq_info_list_free_null(struct pslist **sl_ptr);
+
 #endif	/* _cq_h_ */
 
 /* vi: set ts=4: */
