@@ -1,5 +1,6 @@
 /*
  * Copyright (c) 2002-2003, Richard Eckart
+ * Copyright (c) 2013 Raphael Manfredi
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -21,11 +22,25 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup lib
+ * @file
+ *
+ * Event mangement & dispatching logic.
+ *
+ * @author Richard Eckart
+ * @date 2002-2003
+ * @author Raphael Manfredi
+ * @date 2013
+ */
+
 #ifndef _event_h_
 #define _event_h_
 
 #include "common.h"
 
+#include "pslist.h"
+#include "mutex.h"
 #include "tm.h"
 
 typedef enum frequency_type {
@@ -34,7 +49,7 @@ typedef enum frequency_type {
 } frequency_t;
 
 struct subscriber {
-    GCallback           cb;
+    callback_fn_t       cb;
     enum frequency_type f_type;
     uint32              f_interval;
     time_t              last_call;
@@ -42,22 +57,18 @@ struct subscriber {
 
 typedef struct event {
     const char *name;
-    uint32      triggered_count;
-    GSList     *subscribers;
+    uint32     triggered_count;
+    pslist_t   *subscribers;
+	mutex_t    lock;
+	bool       destroyed;
 } event_t;
 
 struct event *event_new(const char *name);
 
-#define event_destroy(evt) G_STMT_START {                          \
-    real_event_destroy(evt);                                       \
-	/* Event not freed, allocated via omalloc() */                 \
-} G_STMT_END
-
-void real_event_destroy(struct event *evt);
-
+void event_destroy(struct event *evt);
 void event_add_subscriber(
-    struct event *evt, GCallback cb, frequency_t t, uint32 interval);
-void event_remove_subscriber(struct event *evt, GCallback cb);
+    struct event *evt, callback_fn_t cb, frequency_t t, uint32 interval);
+void event_remove_subscriber(struct event *evt, callback_fn_t cb);
 
 uint event_subscriber_count(struct event *evt);
 bool event_subscriber_active(struct event *evt);
@@ -74,17 +85,18 @@ bool event_subscriber_active(struct event *evt);
 
 #define event_trigger(ev, callback) G_STMT_START {				  		 	\
 	struct {																\
-		GSList *sl;											 			 	\
+		pslist_t *sl;										 			 	\
 		event_t *evt;										   				\
 		struct subscriber *s;												\
 		time_t now;									   					 	\
 		bool t;																\
 	} vars_;																\
 																			\
+	mutex_lock(&(ev)->lock);												\
 	vars_.evt = (ev);														\
 	vars_.now = (time_t) -1;												\
 	vars_.sl = vars_.evt->subscribers;										\
-	for (/* NOTHING */; vars_.sl; vars_.sl = g_slist_next(vars_.sl)) {		\
+	for (/* NOTHING */; vars_.sl; vars_.sl = pslist_next(vars_.sl)) {		\
 		vars_.s = vars_.sl->data;											\
 		vars_.t = 0 == vars_.s->f_interval;									\
 		if (!vars_.t) {														\
@@ -113,27 +125,9 @@ bool event_subscriber_active(struct event *evt);
 		}																	\
 	}																		\
 	vars_.evt->triggered_count++;											\
+	mutex_unlock(&(ev)->lock);												\
 } G_STMT_END
 
-struct hikset;
-
-struct event_table {
-    struct hikset *events;
-};
-
-struct event_table *event_table_new(void);
-
-#define event_table_destroy(t) G_STMT_START {                      \
-    real_event_table_destroy(t);                                   \
-    WFREE_NULL(t, sizeof *t);                                      \
-} G_STMT_END
-void real_event_table_destroy(struct event_table *t, bool cleanup);
-
-
-void event_table_add_event(struct event_table *t, struct event *evt);
-void event_table_remove_event(struct event_table *t, struct event *evt);
-void event_table_remove_all(struct event_table *t);
-
 #endif	/* _event_h_ */
 
 /* vi: set ts=4 sw=4 cindent: */
