@@ -52,6 +52,7 @@ void *hash_list_next(hash_list_t *, const void *key);
 void *hash_list_previous(hash_list_t *, const void *key);
 void hash_list_clear(hash_list_t *hl);
 unsigned hash_list_length(const hash_list_t *);
+size_t hash_list_count(const hash_list_t *);
 struct plist *hash_list_list(hash_list_t *) WARN_UNUSED_RESULT;
 void hash_list_sort(hash_list_t *, cmp_fn_t);
 void hash_list_sort_with_data(hash_list_t *, cmp_data_fn_t, void *);
@@ -74,7 +75,7 @@ void *hash_list_iter_remove(hash_list_iter_t *iter);
 
 bool hash_list_find(hash_list_t *, const void *key, const void **orig_key);
 bool hash_list_contains(hash_list_t *, const void *key);
-const void *hash_list_lookup(hash_list_t *hl, const void *key);
+void *hash_list_lookup(hash_list_t *hl, const void *key);
 void hash_list_foreach(const hash_list_t *, data_fn_t, void *);
 size_t hash_list_foreach_remove(hash_list_t *, data_rm_fn_t, void *);
 
