@@ -25,7 +25,7 @@
  * @ingroup lib
  * @file
  *
- * Needs brief description here.
+ * Event listening interface.
  *
  * @author Richard Eckart
  * @date 2001-2003
@@ -36,6 +36,9 @@
 
 #include "common.h"
 
+#include "spinlock.h"
+#include "pslist.h"
+
 /**
  * OVERVIEW
  *
@@ -139,31 +142,42 @@
  * events triggered must be clearly known.
  */
 
-typedef GSList *listeners_t;
+typedef pslist_t *listeners_t;
+
+spinlock_t *listener_get_lock(const char *name);
 
 #define LISTENER_ADD(signal, callback) 										\
 G_STMT_START {																\
 	void *p = func_to_pointer(callback);									\
+	spinlock_t *lock = listener_get_lock(STRINGIFY(signal));				\
 	g_assert(NULL != p);				 									\
-	CAT2(signal,_listeners) = g_slist_append(CAT2(signal,_listeners), p);	\
+	spinlock(lock);															\
+	CAT2(signal,_listeners) = pslist_append(CAT2(signal,_listeners), p);	\
+	spinunlock(lock);														\
 } G_STMT_END
 
 #define LISTENER_REMOVE(signal, callback)									\
 G_STMT_START {																\
 	void *p = func_to_pointer(callback);									\
+	spinlock_t *lock = listener_get_lock(STRINGIFY(signal));				\
 	g_assert(NULL != p);													\
-	CAT2(signal,_listeners) = g_slist_remove(CAT2(signal,_listeners), p);	\
+	spinlock(lock);															\
+	CAT2(signal,_listeners) = pslist_remove(CAT2(signal,_listeners), p);	\
+	spinunlock(lock);														\
 } G_STMT_END
 
 #define LISTENER_EMIT(signal, params)										\
 G_STMT_START {																\
-	GSList *sl;													 			\
-	for (sl = CAT2(signal,_listeners); sl != NULL; sl = g_slist_next(sl)) { \
+	pslist_t *sl;												 			\
+	spinlock_t *lock = listener_get_lock(STRINGIFY(signal));				\
+	spinlock(lock);															\
+	for (sl = CAT2(signal,_listeners); sl != NULL; sl = pslist_next(sl)) {	\
 		CAT2(signal,_listener_t) fn;										\
 		g_assert(NULL != sl->data);	  										\
-		fn = (CAT2(signal,_listener_t)) cast_pointer_to_func(sl->data);	\
+		fn = (CAT2(signal,_listener_t)) cast_pointer_to_func(sl->data);		\
 		fn params;															\
 	}																		\
+	spinunlock(lock);														\
 } G_STMT_END
 
 #endif /* _listener_h_ */
