@@ -70,6 +70,7 @@
 #include "halloc.h"
 #include "hashing.h"		/* For string_mix_hash() and string_eq() */
 #include "hashtable.h"
+#include "misc.h"			/* For CONST_STRLEN() */
 #include "offtime.h"
 #include "once.h"
 #include "signal.h"
@@ -108,6 +109,19 @@ static once_flag_t log_okey_inited;
 static thread_key_t log_strkey = THREAD_KEY_INIT;
 static once_flag_t log_strkey_inited;
 
+/**
+ * How large is the string holding the optional PID to log (e.g " [12345]").
+ */
+#define LOG_PIDLEN		(ULONG_DEC_BUFLEN + CONST_STRLEN(" []"))
+
+/**
+ * Length of a buffer able to hold the formatted time plus optionally
+ * the PID of the process.
+ */
+#define LOG_TIME_BUFLEN	(CRASH_TIME_BUFLEN + LOG_PIDLEN)
+
+static char logpid[LOG_PIDLEN];	/**< If non-empty, also log process PID */
+
 /**
  * A Log file we manage.
  */
@@ -224,12 +238,27 @@ static struct logfile logfile[LOG_MAX_FILES];
 		G_LIKELY(log_inited) ? logfile[LOG_STDERR].fd : STDERR_FILENO)
 
 /**
- * This is used to detect recurstions.
+ * This is used to detect recursions.
  */
 static volatile sig_atomic_t in_safe_handler;	/* in s_logv() */
+static bool log_crashing;
 
 static const char DEV_NULL[] = "/dev/null";
 
+/**
+ * Make sure all logging routines are always using a raw log.
+ *
+ * Also a copy of all the logs is made to stdout automatically.
+ *
+ * This is used during emergency crashing conditions to ensure logging is
+ * always going to use a safe logging mode.
+ */
+void G_GNUC_COLD
+log_crash_mode(void)
+{
+	log_crashing = TRUE;
+}
+
 /**
  * @return pre-allocated chunk for allocating memory when no malloc() wanted.
  */
@@ -647,7 +676,7 @@ log_fprint(enum log_file which, const struct tm *ct, long usec,
 	str_t *ls;
 	ssize_t w;
 
-#define FORMAT_STR	"%02d-%02d-%02d %.02d:%.02d:%.02d.%03ld (%s)%s%s: %s\n"
+#define FORMAT_STR	"%02d-%02d-%02d %.02d:%.02d:%.02d.%03ld%s (%s)%s%s: %s\n"
 
 	log_file_check(which);
 
@@ -677,7 +706,7 @@ log_fprint(enum log_file which, const struct tm *ct, long usec,
 	str_printf(ls, FORMAT_STR,
 		(TM_YEAR_ORIGIN + ct->tm_year) % 100,
 		ct->tm_mon + 1, ct->tm_mday,
-		ct->tm_hour, ct->tm_min, ct->tm_sec, usec / 1000, tprefix,
+		ct->tm_hour, ct->tm_min, ct->tm_sec, usec / 1000, logpid, tprefix,
 		(level & G_LOG_FLAG_RECURSION) ? " [RECURSIVE]" : "",
 		(level & G_LOG_FLAG_FATAL) ? " [FATAL]" : "",
 		msg);
@@ -734,6 +763,35 @@ log_prefix(GLogLevelFlags level)
 	}
 }
 
+/**
+ * Fill supplied buffer with the current time formatted as yy-mm-dd HH:MM:SS.sss
+ * and optionally the process PID, if configured to do so.
+ *
+ * The buffer should be at least LOG_TIME_BUFLEN bytes.
+ *
+ * @param buf		buffer where current time is formatted
+ * @param size		length of buffer
+ */
+static void
+log_time(char *buf, size_t size)
+{
+	crash_time(buf, size);
+	clamp_strcat(buf, size, logpid);
+}
+
+/**
+ * Same as log_time() but uses cached time, and therefore does not take locks.
+ *
+ * @param buf		buffer where current time is formatted
+ * @param size		length of buffer
+ */
+static void
+log_time_cached(char *buf, size_t size)
+{
+	crash_time_cached(buf, size);
+	clamp_strcat(buf, size, logpid);
+}
+
 /**
  * Abort and make sure we never return.
  */
@@ -743,6 +801,27 @@ log_abort(void)
 	static void *log_stack[STACKTRACE_DEPTH_MAX];
 	size_t count;
 
+	/*
+	 * If we have already generated a crash log and we are running supervised,
+	 * it is time to exit: we're looping into errors.
+	 */
+
+	if (crash_is_logged() && crash_is_supervised()) {
+		DECLARE_STR(3);
+		char time_buf[LOG_TIME_BUFLEN];
+
+		log_time(time_buf, sizeof time_buf);
+		print_str(time_buf);							/* 0 */
+		print_str(" (CRITICAL): crash log generated");	/* 1 */
+		print_str(", good bye.\n");						/* 2 */
+
+		log_flush_err_atomic();
+		if (log_stdout_is_distinct())
+			log_flush_out_atomic();
+
+		_exit(EXIT_FAILURE);	/* Immediate exit */
+	}
+
 	/*
 	 * It may be difficult to backtrace the stack past the signal handler
 	 * which is going to be invoked by raise(), hence save a copy of the
@@ -772,10 +851,10 @@ log_abort(void)
 
 	{
 		DECLARE_STR(3);
-		char time_buf[CRASH_TIME_BUFLEN];
+		char time_buf[LOG_TIME_BUFLEN];
 
-		crash_time(time_buf, sizeof time_buf);
-		print_str(time_buf);	/* 0 */
+		log_time(time_buf, sizeof time_buf);
+		print_str(time_buf);								/* 0 */
 		print_str(" (CRITICAL): back from raise(SIGBART)"); /* 1 */
 		print_str(" -- invoking crash_handler()\n");		/* 2 */
 		log_flush_err_atomic();
@@ -791,8 +870,8 @@ log_abort(void)
 		 */
 
 		rewind_str(0);
-		crash_time(time_buf, sizeof time_buf);
-		print_str(time_buf);	/* 0 */
+		log_time(time_buf, sizeof time_buf);
+		print_str(time_buf);			/* 0 */
 		print_str(" (CRITICAL): back from crash_handler()"); /* 1 */
 		print_str(" -- exiting\n");		/* 2 */
 		log_flush_err_atomic();
@@ -831,7 +910,7 @@ s_rawlogv(GLogLevelFlags level, bool raw, bool copy,
 {
 	char data[LOG_MSG_MAXLEN];
 	DECLARE_STR(11);
-	char time_buf[CRASH_TIME_BUFLEN];
+	char time_buf[LOG_TIME_BUFLEN];
 	const char *prefix;
 	unsigned stid;
 
@@ -863,10 +942,10 @@ s_rawlogv(GLogLevelFlags level, bool raw, bool copy,
 		stid = thread_safe_small_id();
 		if (THREAD_UNKNOWN_ID == stid)
 			stid = 0;
-		crash_time_cached(time_buf, sizeof time_buf);	/* Raw, no locks! */
+		log_time_cached(time_buf, sizeof time_buf);	/* Raw, no locks! */
 	} else {
 		stid = thread_small_id();
-		crash_time(time_buf, sizeof time_buf);
+		log_time(time_buf, sizeof time_buf);
 	}
 
 	/*
@@ -907,6 +986,22 @@ s_rawlogv(GLogLevelFlags level, bool raw, bool copy,
 		if (copy && log_stdout_is_distinct())
 			log_flush_out_atomic();
 	}
+
+	/*
+	 * When duplication is configured, write a copy of the message
+	 * without any timestamp and debug level tagging.
+	 */
+
+	if G_UNLIKELY(logfile[LOG_STDERR].duplicate) {
+		int fd = logfile[LOG_STDERR].crash_fd;
+		iovec_t iov[2];
+		iovec_set(&iov[0], data, clamp_strlen(data, sizeof data));
+		iovec_set(&iov[1], "\n", 1);
+		if (raw)
+			IGNORE_RESULT(writev(fd, iov, G_N_ELEMENTS(iov)));
+		else
+			atio_writev(fd, iov, G_N_ELEMENTS(iov));
+	}
 }
 
 /**
@@ -923,9 +1018,10 @@ void
 s_minilogv(GLogLevelFlags level, bool copy, const char *fmt, va_list args)
 {
 	int saved_errno;
+	bool crashing = log_crashing;
 
 	saved_errno = errno;
-	s_rawlogv(level, FALSE, copy, fmt, args);
+	s_rawlogv(level, crashing, copy || crashing, fmt, args);
 	errno = saved_errno;
 }
 
@@ -1013,6 +1109,11 @@ s_logv(logthread_t *lt, GLogLevelFlags level, const char *format, va_list args)
 	if (G_UNLIKELY(logfile[LOG_STDERR].disabled))
 		return;
 
+	if G_UNLIKELY(log_crashing) {
+		s_rawlogv(level, TRUE, TRUE, format, args);
+		return;
+	}
+
 	/*
 	 * Detect recursion, but don't make it fatal.
 	 */
@@ -1028,14 +1129,14 @@ s_logv(logthread_t *lt, GLogLevelFlags level, const char *format, va_list args)
 
 	if G_UNLIKELY(recursing) {
 		DECLARE_STR(9);
-		char time_buf[CRASH_TIME_BUFLEN];
+		char time_buf[LOG_TIME_BUFLEN];
 		const char *caller;
 		bool copy;
 
 		stid = NULL == lt ? thread_small_id() : lt->stid;
 		caller = stacktrace_caller_name(2);	/* Could log, so pre-compute */
 
-		crash_time(time_buf, sizeof time_buf);
+		log_time(time_buf, sizeof time_buf);
 		print_str(time_buf);		/* 0 */
 		print_str(" (CRITICAL");	/* 1 */
 		if (0 != stid) {
@@ -1095,9 +1196,9 @@ s_logv(logthread_t *lt, GLogLevelFlags level, const char *format, va_list args)
 
 	if G_UNLIKELY(NULL == msg) {
 		DECLARE_STR(6);
-		char time_buf[CRASH_TIME_BUFLEN];
+		char time_buf[LOG_TIME_BUFLEN];
 
-		crash_time(time_buf, sizeof time_buf);
+		log_time(time_buf, sizeof time_buf);
 		print_str(time_buf);	/* 0 */
 		print_str(" (CRITICAL): no memory to format string \""); /* 1 */
 		print_str(format);		/* 2 */
@@ -1126,9 +1227,9 @@ s_logv(logthread_t *lt, GLogLevelFlags level, const char *format, va_list args)
 
 	{
 		DECLARE_STR(11);
-		char time_buf[CRASH_TIME_BUFLEN];
+		char time_buf[LOG_TIME_BUFLEN];
 
-		crash_time(time_buf, sizeof time_buf);
+		log_time(time_buf, sizeof time_buf);
 		print_str(time_buf);	/* 0 */
 		print_str(" (");		/* 1 */
 		print_str(prefix);		/* 2 */
@@ -1217,13 +1318,16 @@ log_check_recursive(const char *format, va_list ap)
 {
 	static int recursive;
 	static char buf[LOG_MSG_MAXLEN];
+	static int stid;
+	int depth;
 
-	atomic_int_inc(&recursive);
+	depth = atomic_int_inc(&recursive);
 
-	if (1 == recursive) {
+	if (0 == depth) {
 		str_vbprintf(buf, sizeof buf, format, ap);
+		stid = thread_safe_small_id();
 		return FALSE;
-	} else if (2 == recursive) {
+	} else if (1 == depth) {
 		/*
 		 * Ensure we're not losing previous error in case we did not go
 		 * far enough, but flag the string as being from a previous error
@@ -1231,13 +1335,13 @@ log_check_recursive(const char *format, va_list ap)
 		 */
 		crash_set_error("previous error: ");
 		crash_append_error(buf);
-		s_minicrit("error occurred whilst processing former error:");
+		s_miniwarn("error whilst processing error from thread #%d:", stid);
 		s_miniinfo("previous error: %s", buf);
 		return TRUE;
-	} else if (3 == recursive) {
-		s_minicrit("recursive or concurrent error, aborting");
+	} else if (2 == depth) {
+		s_rawwarn("recursive or concurrent error, aborting");
 		log_abort();
-	} else if (4 == recursive) {
+	} else if (3 == depth) {
 		abort();
 	} else {
 		_exit(EXIT_FAILURE);
@@ -1534,7 +1638,7 @@ s_minierror(const char *format, ...)
 	static int recursion;
 	va_list args;
 	char data[LOG_MSG_MAXLEN];
-	char time_buf[CRASH_TIME_BUFLEN];
+	char time_buf[LOG_TIME_BUFLEN];
 	DECLARE_STR(6);
 	bool recursing;
 
@@ -1544,7 +1648,7 @@ s_minierror(const char *format, ...)
 	str_vbprintf(data, sizeof data, format, args);
 	va_end(args);
 
-	crash_time(time_buf, sizeof time_buf);
+	log_time(time_buf, sizeof time_buf);
 	print_str(time_buf);					/* 0 */
 	print_str(" (ERROR)");					/* 1 */
 	if (recursing)
@@ -1581,7 +1685,7 @@ s_rawcrit(const char *format, ...)
 	s_rawlogv(G_LOG_LEVEL_CRITICAL, TRUE, TRUE, format, args);
 	va_end(args);
 
-	s_stacktrace(in_signal_handler, 1);		/* Copied to stdout if different */
+	s_stacktrace(in_signal_handler, 1);	/* Copied to stdout if different */
 }
 
 /**
@@ -2100,9 +2204,9 @@ log_reopen(enum log_file which)
 			s_critical("freopen(\"%s\", \"a\", ...) failed: %m", lf->path);
 		} else if (is_valid_fd(fd)) {
 			DECLARE_STR(8);
-			char time_buf[CRASH_TIME_BUFLEN];
+			char time_buf[LOG_TIME_BUFLEN];
 
-			crash_time(time_buf, sizeof time_buf);
+			log_time(time_buf, sizeof time_buf);
 			print_str(time_buf);	/* 0 */
 			print_str(" (CRITICAL): cannot freopen() stderr to "); /* 1 */
 			print_str(lf->path);	/* 2 */
@@ -2221,6 +2325,19 @@ log_set_disabled(enum log_file which, bool disabled)
 	logfile[which].disabled = disabled;
 }
 
+/**
+ * Enable or disable PID logging.
+ */
+void
+log_show_pid(bool enabled)
+{
+	if (enabled) {
+		str_bprintf(logpid, sizeof logpid, " [%lu]", (ulong) getpid());
+	} else {
+		logpid[0] = '\0';	/* Empty string, shows nothing */
+	}
+}
+
 /**
  * Record duplicate file descriptor where messages will also be written
  * albeit without any prefixing.
@@ -2430,7 +2547,7 @@ log_atoms_inited(void)
  * Record formatting string to be used to format messages when crashing.
  */
 void
-log_crashing(struct str *str)
+log_crashing_str(struct str *str)
 {
 	g_assert(str != NULL);
 
