@@ -1301,7 +1301,7 @@ malloc_log_block(const void *k, void *v, void *leaksort)
 
 #ifdef MALLOC_TIME
 	str_bprintf(ago, sizeof ago, " [%s]",
-		short_time(delta_time(tm_time(), b->ttime)));
+		short_time_ascii(delta_time(tm_time(), b->ttime)));
 #else
 	ago[0] = '\0';
 #endif	/* MALLOC_TIME */
@@ -1394,7 +1394,7 @@ malloc_log_real_block(const void *k, void *v, void *leaksort)
 
 #ifdef MALLOC_TIME
 	str_bprintf(ago, sizeof ago, " [%s]",
-		short_time(delta_time(tm_time(), rb->atime)));
+		short_time_ascii(delta_time(tm_time(), rb->atime)));
 #else
 	ago[0] = '\0';
 #endif	/* MALLOC_TIME */
@@ -2074,6 +2074,33 @@ strconcat_track(const char *file, int line, const char *s, ...)
 	return malloc_record(o, strlen(o) + 1, FALSE, file, line);
 }
 
+/**
+ * Perform string concatenation, returning newly allocated string.
+ */
+char *
+strconcat_v_track(const char *file, int line, const char *s, va_list ap)
+{
+	char *o;
+
+	o = m_strconcatv(s, ap);
+
+	/*
+	 * FIXME:
+	 *
+	 * m_strconcatv() uses real_malloc(), but we cannot mark we own this
+	 * block as there is no malloc_header structure put in case we're
+	 * compiled with MALLOC_SAFE_HEAD.
+	 *
+	 * To be able to do that, we need to have more flags in the block
+	 * and be able to pass them on to malloc_record (i.e. it must not just
+	 * take TRUE/FALSE but a set of flags) so that we can tell the lower
+	 * layers whether a block allocated through real_malloc() has an
+	 * additional malloc_header in front of the data.
+	 */
+
+	return malloc_record(o, strlen(o) + 1, FALSE, file, line);
+}
+
 /**
  * Perform printf into newly allocated string.
  */
@@ -2796,7 +2823,7 @@ alloc_dump(FILE *f, bool total)
 
 	now = tm_time();
 	fprintf(f, "--- distinct allocation spots found: %d at %s\n",
-		count, short_time(delta_time(now, init_time)));
+		count, short_time_ascii(delta_time(now, init_time)));
 
 	filler.stats = real_malloc(sizeof(struct stats *) * count);
 	filler.count = count;
@@ -2817,7 +2844,7 @@ alloc_dump(FILE *f, bool total)
 
 	fprintf(f, "--- summary by decreasing %s allocation size %s %s:\n",
 		total ? "total" : "incremental", total ? "at" : "after",
-		short_time(delta_time(now, total ? init_time : reset_time)));
+		short_time_ascii(delta_time(now, total ? init_time : reset_time)));
 	stats_array_dump(f, &filler);
 
 	/*
@@ -2832,7 +2859,7 @@ alloc_dump(FILE *f, bool total)
 
 	fprintf(f, "--- summary by decreasing %s residual memory size %s %s:\n",
 		total ? "total" : "incremental", total ? "at" : "after",
-		short_time(now - (total ? init_time : reset_time)));
+		short_time_ascii(now - (total ? init_time : reset_time)));
 	stats_array_dump(f, &filler);
 
 	/*
@@ -2848,11 +2875,11 @@ alloc_dump(FILE *f, bool total)
 			stats_total_residual_cmp);
 
 		fprintf(f, "--- summary by decreasing %s residual memory size %s %s:\n",
-			"total", "at", short_time(delta_time(now, init_time)));
+			"total", "at", short_time_ascii(delta_time(now, init_time)));
 		stats_array_dump(f, &filler);
 	}
 
-	fprintf(f, "--- end summary at %s\n", short_time(now - init_time));
+	fprintf(f, "--- end summary at %s\n", short_time_ascii(now - init_time));
 
 	real_free(filler.stats);
 }
@@ -2884,7 +2911,7 @@ alloc_reset(FILE *f, bool total)
 	hash_table_foreach(stats, stats_reset, NULL);
 
 	fprintf(f, "--- incremental allocation stats reset after %s.\n",
-		short_time(now - reset_time));
+		short_time_ascii(now - reset_time));
 
 	reset_time = now;
 }
@@ -2989,11 +3016,10 @@ malloc_init_vtable(void)
 	}
 #else	/* !MALLOC_VTABLE */
 	/*
-	 * On Windows, when xmalloc() is actually malloc(), redirect all glib
-	 * memory allocation to malloc() / free().
+	 * On Windows, redirect all glib memory allocation to xmalloc() / xfree().
 	 */
 
-	if (is_running_on_mingw() && xmalloc_is_malloc()) {
+	if (is_running_on_mingw()) {
 		static GMemVTable vtable;
 
 #if GLIB_CHECK_VERSION(2,0,0)
@@ -3001,9 +3027,9 @@ malloc_init_vtable(void)
 		putenv(variable);
 #endif	/* GLib >= 2.0.0 */
 
-		vtable.malloc = malloc;
-		vtable.realloc = realloc;
-		vtable.free = free;
+		vtable.malloc = xmalloc;
+		vtable.realloc = xrealloc;
+		vtable.free = xfree;
 
 		g_mem_set_vtable(&vtable);
 	}
