@@ -34,6 +34,15 @@
 #ifndef _once_h_
 #define _once_h_
 
+/**
+ * Tri-state once initialization flag.
+ */
+typedef enum once_flag {
+	ONCE_F_UNDONE = 0,
+	ONCE_F_PROGRESS = 1,
+	ONCE_F_DONE = 2
+} once_flag_t;
+
 /**
  * Once initialization routine.
  */
@@ -43,7 +52,23 @@ typedef void (*once_fn_t)(void);
  * Public interface.
  */
 
-bool once_run(volatile bool *flag, once_fn_t routine);
+void once_flag_run(once_flag_t *flag, once_fn_t routine);
+bool once_flag_run_safe(once_flag_t *flag, once_fn_t routine);
+
+void once_flag_runwait(once_flag_t *flag, once_fn_t routine);
+bool once_flag_runwait_safe(once_flag_t *flag, once_fn_t routine);
+
+#define ONCE_DONE(f)	(ONCE_F_DONE == (f))
+
+#define ONCE_FLAG_RUN(f, r) G_STMT_START {	\
+	if G_UNLIKELY(!ONCE_DONE((f)))			\
+		once_flag_run(&(f), (r));			\
+} G_STMT_END
+
+#define ONCE_FLAG_RUNWAIT(f, r) G_STMT_START {	\
+	if G_UNLIKELY(!ONCE_DONE((f)))				\
+		once_flag_runwait(&(f), (r));			\
+} G_STMT_END
 
 #endif /* _once_h_ */
 
