@@ -57,10 +57,6 @@
 
 #include "common.h"
 
-#ifdef I_SCHED
-#include <sched.h>
-#endif
-
 #include "rwlock.h"
 
 #include "compat_usleep.h"
@@ -650,6 +646,8 @@ rwlock_init(rwlock_t *rw)
  * It is not necessary to hold the write lock to do this, although one must be
  * careful to not destroy a lock that could be used by another thread.
  *
+ * When called with the write-lock owned, it is automatically unlocked.
+ *
  * Any further attempt to use this lock will cause an assertion failure.
  */
 void
@@ -659,13 +657,23 @@ rwlock_destroy(rwlock_t *rw)
 
 	if (rw->waiters != 0 || rw->readers != 0 || rw->writers != 0) {
 		uint rwait = rw->writers - rw->write_waiters;
+		bool owned = rwlock_is_owned(rw);
+		bool need_carp = TRUE;
 
-		s_carp("destroying rwlock %p with %u reader%s, "
-			"%u writer%s, %u read-waiter%s and %u write-waiter%s",
-			rw, rw->readers, plural(rw->readers),
-			rw->writers, plural(rw->writers),
-			rwait, plural(rwait),
-			rw->write_waiters, plural(rw->write_waiters));
+		if (owned)
+			need_carp = rw->waiters != 0 || rw->readers != 0 || rw->writers > 1;
+
+		if (need_carp) {
+			s_carp("destroying %srwlock %p with %u reader%s, "
+				"%u writer%s, %u read-waiter%s and %u write-waiter%s",
+				owned ? "owned " : "", rw, rw->readers, plural(rw->readers),
+				rw->writers, plural(rw->writers),
+				rwait, plural(rwait),
+				rw->write_waiters, plural(rw->write_waiters));
+		}
+
+		if (owned)
+			rwlock_write_unaccount(rw);
 	}
 
 	rw->magic = RWLOCK_DESTROYED;		/* Now invalid */
@@ -699,6 +707,25 @@ rwlock_is_owned(const rwlock_t *rw)
 	return thread_small_id() == rw->owner;
 }
 
+/**
+ * Is write lock taken by current thread (either read or write)?
+ */
+bool
+rwlock_is_taken(const rwlock_t *rw)
+{
+	rwlock_check(rw);
+
+	if (thread_lock_holds(rw))
+		return TRUE;
+
+	if G_UNLIKELY(rwlock_pass_through) {
+		thread_check_suspended();
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 /**
  * Check whether lock is used.
  */
