@@ -65,10 +65,14 @@ const char *signal_name(int signo);
 bool signal_in_handler(void);
 struct ckhunk *signal_chunk(void);
 void signal_unblock(int signo);
-void signal_abort(void) G_GNUC_NORETURN;
+void signal_abort(void);
 size_t signal_stack_allocate(void **base_ptr);
 bool signal_stack_free(void **base_ptr);
 
+void signal_crashing(void);
+void signal_uncrashing(void);
+int signal_in_exception(void);
+
 bool signal_enter_critical(sigset_t *oset);
 void signal_leave_critical(const sigset_t *oset);
 
