@@ -248,6 +248,8 @@ void spinlock_loop(volatile spinlock_t *s,
 	spinlock_deadlock_cb_t deadlock, spinlock_deadlocked_cb_t deadlocked,
 	const char *file, unsigned line);
 
+void spinlock_set_owner_external(spinlock_t *, const char *, unsigned);
+
 #endif /* SPINLOCK_SOURCE || MUTEX_SOURCE */
 
 #ifdef THREAD_SOURCE
@@ -272,9 +274,12 @@ static inline bool NON_NULL_PARAM((1))
 spinlock_is_held(const spinlock_t *s)
 {
 #ifdef SPINLOCK_OWNER_DEBUG
-	if (thread_small_id() != s->stid)
+	if (
+		(uint8) THREAD_UNKNOWN_ID != s->stid &&
+		(uint8) thread_safe_small_id() != s->stid
+	)
 		return FALSE;
-#endif
+#endif	/* SPINLOCK_OWNER_DEBUG */
 
 	/* Make this fast, no assertion on the spinlock validity */
 	return s->lock != 0 || spinlock_in_crash_mode();
