@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996-2000, 2007, 2010-2013 Raphael Manfredi
+ * Copyright (c) 1996-2000, 2007, 2010-2015 Raphael Manfredi
  *
  * This code given by Raphael Manfredi, extracted from his fm2html package.
  * Also contains some code borrowed from Perl: routine str_vncatf().
@@ -39,7 +39,7 @@
  * Memory must be released with hfree().
  *
  * @author Raphael Manfredi
- * @date 1996-2000, 2007, 2010-2013
+ * @date 1996-2000, 2007, 2010-2015
  */
 
 #include "common.h"
@@ -505,6 +505,19 @@ str_free(str_t *str)
 	str->s_size = 0;
 }
 
+/**
+ * Discard the str_t structure, freeing string data and making object invalid.
+ *
+ * This is to be used on static str_t structures, when we want to free the
+ * allocated memory and prevent any further usage of the structure.
+ */
+void
+str_discard(str_t *str)
+{
+	str_free(str);
+	str->s_magic = 0;
+}
+
 /**
  * Destroy string held within the str_t structure, then the structure.
  */
@@ -516,8 +529,7 @@ str_destroy(str_t *str)
 	g_assert_log(str->s_flags & STR_OBJECT,
 		"%s(): called on \"static\" string object", G_STRFUNC);
 
-	str_free(str);
-	str->s_magic = 0;
+	str_discard(str);
 	WFREE(str);
 }
 
@@ -1417,6 +1429,217 @@ str_escape(str_t *str, char c, char e)
 	}
 }
 
+/**
+ * Translate algebric offset into physical offset, with specific off-limit
+ * indication.
+ *
+ * A negative offset is interpreted starting from the end of the string,
+ * with -1 being the last character.
+ *
+ * When the physical offset falls before the start of the string, -1 is
+ * returned.  If it falls beyond the end of the string, length is returned
+ * (i.e. one character beyond the actual end).
+ */
+static ssize_t
+str_offset(const str_t *s, ssize_t offset)
+{
+	ssize_t i;
+
+	i = offset >= 0 ? offset : (ssize_t) s->s_len + offset;
+	if (i < 0)
+		i = -1;
+	else if (UNSIGNED(i) > s->s_len)
+		i = s->s_len;
+
+	return i;
+}
+
+/**
+ * Translate algebric offset into physical offset within string.
+ *
+ * A negative offset is interpreted starting from the end of the string,
+ * with -1 being the last character.
+ *
+ * The offset is always adjusted to fall within the string, rounding it up
+ * to 0 when it is negative and points before the start, or down to the last
+ * character in the string when it is positive and points beyond the end.
+ *
+ * @return a safe offset, between 0 (inclusive) and length (exclusive).
+ */
+static size_t
+str_offset_safe(const str_t *s, ssize_t offset)
+{
+	size_t i;
+
+	if (offset >= 0) {
+		size_t pos = offset;
+		i = pos >= s->s_len ? (0 == s->s_len ? 0 : s->s_len - 1) : pos;
+	} else {
+		size_t pos = s->s_len + offset;
+		i = pos >= s->s_len ? 0 : pos;
+	}
+
+	return i;
+}
+
+/**
+ * Look for character ``c'' in string, starting at given offset.
+ *
+ * A negative offset is interpreted starting from the end of the string,
+ * with -1 being the last character.
+ *
+ * @return the offset from the start of the string where character is found,
+ * or -1 if not found.
+ */
+ssize_t
+str_chr_at(const str_t *s, int c, ssize_t offset)
+{
+	size_t len, i;
+	ssize_t pos;
+	const char *p;
+
+	str_check(s);
+
+	len = s->s_len;
+	i = pos = str_offset(s, offset);
+	if (pos < 0)
+		i = 0;
+	else if (UNSIGNED(pos) >= len)
+		return -1;
+
+	for (p = s->s_data + i; i < len; i++) {
+		if G_UNLIKELY(*p++ == c)
+			return i;
+	}
+
+	return -1;
+}
+
+/**
+ * Look for character ``c'' in string, starting at position 0.
+ *
+ * @return the offset from the start of the string where character is found,
+ * or -1 if not found.
+ */
+ssize_t
+str_chr(const str_t *s, int c)
+{
+	return str_chr_at(s, c, 0);
+}
+
+/**
+ * Backward look for character ``c'' in string, starting at given offset.
+ *
+ * A negative offset is interpreted starting from the end of the string,
+ * with -1 being the last character.
+ *
+ * @return the offset from the start of the string where character is found,
+ * or -1 if not found.
+ */
+ssize_t
+str_rchr_at(const str_t *s, int c, ssize_t offset)
+{
+	size_t len, i;
+	ssize_t pos;
+	const char *p;
+
+	str_check(s);
+
+	len = s->s_len;
+	i = pos = str_offset(s, offset);
+	if (pos < 0)
+		return -1;
+	else if (UNSIGNED(pos) >= len)
+		i = len - 1;
+
+	for (p = s->s_data + i++; i != 0; i--) {
+		if G_UNLIKELY(*p-- == c)
+			return i - 1;
+	}
+
+	return -1;
+}
+
+/**
+ * Backward look for character ``c'' in string, starting at end of string.
+ *
+ * @return the offset from the start of the string where character is found,
+ * or -1 if not found.
+ */
+ssize_t
+str_rchr(const str_t *s, int c)
+{
+	return str_rchr_at(s, c, -1);
+}
+
+/**
+ * Generate a new string from the characters within the specified range
+ * in the given string.  Range is inclusive on both sides.
+ *
+ * A negative offset is interpreted starting from the end of the string,
+ * with -1 being the last character.
+ *
+ * @return a new string object
+ */
+str_t *
+str_slice(const str_t *s, ssize_t from, ssize_t to)
+{
+	str_t *n;
+	size_t len, start, end;
+
+	str_check(s);
+
+	start = str_offset_safe(s, from);
+	end   = str_offset_safe(s, to);
+
+	len = start <= end ? end - start + 1 : 0;
+	if (0 == s->s_len)
+		len = 0;
+	n = str_new(len + 1);	/* Allow for trailing NUL in str_2c() */
+	n->s_len = len;
+	memcpy(n->s_data, s->s_data + start, len);
+
+	return n;
+}
+
+/**
+ * Generate a new string that is a substring of characters from given
+ * string, starting at a given offset and of specified length.
+ *
+ * A negative offset is interpreted starting from the end of the string,
+ * with -1 being the last character.
+ *
+ * When the length specified would include characters beyond the end of
+ * the string, the length is truncated to the actual string length.
+ *
+ * @return a new string object
+ */
+str_t *
+str_substr(const str_t *s, ssize_t from, size_t length)
+{
+	str_t *n;
+	size_t len, start;
+
+	str_check(s);
+	g_assert(size_is_non_negative(length));
+
+	start = str_offset_safe(s, from);
+	if (0 == s->s_len) {
+		len = 0;
+	} else {
+		size_t end = start + length;	/* One char beyond */
+		if (end > s->s_len)
+			end = s->s_len;
+		len = start <= end ? end - start : 0;
+	}
+
+	n = str_new(len + 1);	/* Allow for trailing NUL in str_2c() */
+	n->s_len = len;
+	memcpy(n->s_data, s->s_data + start, len);
+
+	return n;
+}
+
 /**
  * Round up floating-point mantissa, with leading extra carry digit.
  *
@@ -3126,6 +3349,94 @@ str_tprintf(char *dst, size_t size, const char *fmt, ...)
 	str_putc(&str, '\0');
 }
 
+/**
+ * Fix the exponent part of a formatted double.
+ *
+ * On Windows, and maybe on other systems, snprintf() formats exponents with
+ * 3 digits, whereas our str_vncatf() routine uses 2 digits.
+ *
+ * This routine normalizes exponents to 2 digits, for the purpose of
+ * avoiding spurious discrepancies reports when issues a verbose str_test().
+ */
+static void G_GNUC_COLD
+str_test_fix_exponent(char *std)
+{
+	char *p = std;
+	int c;
+	bool has_leading_space;
+
+	has_leading_space = ' ' == *p;
+
+	while ((c = *p++)) {
+		int a;
+
+		if ('e' == c || 'E' == c) {
+			char *extra;
+
+			c = *p++;
+			if ('-' != c && '+' != c)
+				goto error;	/* Not in an exponent */
+
+			extra = p;		/* Extra '0' spot, if any present */
+			c = *p++;
+			if (c != '0')
+				break;		/* Nothing to fix */
+
+			c = *p++;
+			if (!is_ascii_digit(c))
+				goto error;	/* Only one digit after exponent, that's bad */
+
+			c = *p++;
+			if (!is_ascii_digit(c))
+				break;		/* Out of number, in formatted space now */
+
+			/*
+			 * Ah, reached a third digit with an expoenent starting with '0'
+			 * at ``extra'' in the string.  Move everything back by one char.
+			 */
+
+			a = *p;		/* Character after the number */
+
+			g_assert(a == '\0' || is_ascii_space(a));
+
+			p = extra;
+			while (1) {
+				*p = *(p + 1);
+				if (a != '\0' && a == *p)
+					break;			/* Leave trailing spaces intact */
+				if ('\0' == *p++)
+					break;
+			}
+
+			/*
+			 * We removed one digit in the format, so we need to restore
+			 * an additional leading space to compensate for the missing
+			 * character, to not mess-up with the right-justification that
+			 * necessarily occurred during the formatting.
+			 */
+
+			if (has_leading_space) {
+				a = ' ';
+				p = std;
+				while (1) {
+					int t = *p;
+					*p++ = a;
+					if ('\0' == a)
+						break;
+					a = t;
+				}
+			}
+
+			break;			/* Done, string fixed */
+		}
+	}
+
+	return;		/* OK, nothing to fix */
+
+error:
+	s_error("%s(): invalid exponent in \"%s\"", G_STRFUNC, std);
+}
+
 /**
  * Non-regression tests for the str_vncatf() formatting routine.
  *
@@ -3135,7 +3446,7 @@ str_tprintf(char *dst, size_t size, const char *fmt, ...)
  *
  * @return amount of discrepancies found with the system's snprintf().
  */
-G_GNUC_COLD size_t
+size_t G_GNUC_COLD
 str_test(bool verbose)
 {
 #define MLEN		64
@@ -3474,6 +3785,8 @@ str_test(bool verbose)
 			gm_snprintf_unchecked(std, sizeof std,				\
 				t->fmt, t->value);								\
 			std[t->buflen - 1] = '\0';	/* Truncate here */		\
+			if (0 == ptr_cmp(&test_##what##s, &test_doubles))	\
+				str_test_fix_exponent(std);						\
 			gm_snprintf(value, sizeof value, vfmt, t->value);	\
 			if (0 != strcmp(std, buf)) {						\
 				discrepancies++;								\
