@@ -105,6 +105,7 @@
 #define USHRT_DEC_GRP_BUFLEN	GROUPPED_DEC_BUFLEN(unsigned short)
 #define UINT_DEC_GRP_BUFLEN		GROUPPED_DEC_BUFLEN(unsigned int)
 #define ULONG_DEC_GRP_BUFLEN	GROUPPED_DEC_BUFLEN(unsigned long)
+#define FILESIZE_DEC_GRP_BUFLEN	GROUPPED_DEC_BUFLEN(filesize_t)
 
 size_t int32_to_string_buf(int32 v, char *dst, size_t size);
 size_t uint32_to_string_buf(uint32 v, char *dst, size_t size);
@@ -117,6 +118,8 @@ size_t pointer_to_string_buf(const void *ptr, char *dst, size_t size);
 const char *uint32_to_string(uint32);
 const char *uint64_to_string(uint64);
 const char *uint64_to_string2(uint64);
+const char *uint64_to_string3(uint64);
+const char *ulong_to_string(ulong v);
 const char *uint_to_string(unsigned v);
 const char *fileoffset_t_to_string(fileoffset_t);
 const char *size_t_to_string(size_t);
@@ -146,6 +149,7 @@ const char *uint32_to_gstring(uint32);
 const char *uint64_to_gstring(uint64);
 const char *uint_to_gstring(unsigned v);
 const char *size_t_to_gstring(size_t v);
+const char *filesize_to_gstring(filesize_t v);
 
 /*
  * Time string conversions
@@ -155,11 +159,51 @@ const char *short_time_ascii(time_delta_t t);
 size_t compact_time_to_buf(time_delta_t t, char *dst, size_t size);
 const char *compact_time(time_delta_t t);
 const char *compact_time2(time_delta_t t);
+size_t compact_time_ms_to_buf(long t, char *dst, size_t size);
+const char *compact_time_ms(long t);
 const char *short_uptime(time_delta_t s);
 size_t time_locale_to_string_buf(time_t date, char *dst, size_t size);
 size_t time_t_to_string_buf(time_t v, char *dst, size_t size);
 const char *time_t_to_string(time_t);
 
+/*
+ * Miscellaneous stringifications.
+ */
+
+static inline const char *
+bool_to_string(const bool v)
+{
+	return v ? "TRUE" : "FALSE";
+}
+
+/* Plural of most words */
+static inline const char *
+plural(const unsigned long v)
+{
+	return 1 == v ? "" : "s";
+}
+
+/* Plural of words finishing in "y" like "entry" */
+static inline const char *
+plural_y(const unsigned long v)
+{
+	return 1 == v ? "y" : "ies";
+}
+
+/* Plural of words finishing in "ch" like "match" */
+static inline const char *
+plural_es(const unsigned long v)
+{
+	return 1 == v ? "" : "es";
+}
+
+/* Plural of words finishing in "f" like "leaf" */
+static inline const char *
+plural_f(const unsigned long v)
+{
+	return 1 == v ? "f" : "ves";
+}
+
 #endif /* _stringify_h_ */
 
 /* vi: set ts=4 sw=4 cindent: */
