@@ -37,7 +37,7 @@
 #include "common.h"
 
 void tiger_check(void);
-void tiger(gconstpointer data, guint64 length, char hash[24]);
+void tiger(const void *data, uint64 length, char hash[24]);
 
 #endif /* _tiger_h_ */
 /* vi: set ts=4 sw=4 cindent: */
