@@ -58,6 +58,7 @@
 #define T_FIRE	FOURCC_NATIVE('F','I','R','E')
 #define T_FISH	FOURCC_NATIVE('F','I','S','H')
 #define T_FOXY	FOURCC_NATIVE('F','O','X','Y')
+#define T_FSCP  FOURCC_NATIVE('F','S','C','P')
 #define T_FZZN  FOURCC_NATIVE('F','Z','Z','N')
 #define T_GDNA	FOURCC_NATIVE('G','D','N','A')
 #define T_GIFT  FOURCC_NATIVE('G','I','F','T')
@@ -73,6 +74,7 @@
 #define T_GTKG	FOURCC_NATIVE('G','T','K','G')
 #define T_HSLG	FOURCC_NATIVE('H','S','L','G')
 #define T_HUIT  FOURCC_NATIVE('H','U','I','T')
+#define T_HYDR  FOURCC_NATIVE('H','Y','D','R')
 #define T_JHOP  FOURCC_NATIVE('J','H','O','P')
 #define T_JOEY  FOURCC_NATIVE('J','O','E','Y')
 #define T_KIKI  FOURCC_NATIVE('K','I','K','I')
@@ -85,8 +87,10 @@
 #define T_MLDK  FOURCC_NATIVE('M','L','D','K')
 #define T_MMMM	FOURCC_NATIVE('M','M','M','M')
 #define T_MNAP	FOURCC_NATIVE('M','N','A','P')
+#define T_MOOD  FOURCC_NATIVE('M','O','O','D')
 #define T_MRPH	FOURCC_NATIVE('M','R','P','H')
 #define T_MUTE	FOURCC_NATIVE('M','U','T','E')
+#define T_MXIE  FOURCC_NATIVE('M','X','I','E')
 #define T_NAPS	FOURCC_NATIVE('N','A','P','S')
 #define T_NGET  FOURCC_NATIVE('N','G','E','T')
 #define T_NOOG  FOURCC_NATIVE('N','O','O','G')
@@ -95,8 +99,11 @@
 #define T_OPRA	FOURCC_NATIVE('O','P','R','A')
 #define T_OXID  FOURCC_NATIVE('O','X','I','D')
 #define T_PCST  FOURCC_NATIVE('P','C','S','T')
+#define T_PEER  FOURCC_NATIVE('P','E','E','R')
 #define T_PHEX	FOURCC_NATIVE('P','H','E','X')
 #define T_PWRT  FOURCC_NATIVE('P','W','R','T')
+#define T_QAZA  FOURCC_NATIVE('Q','A','Z','A')
+#define T_QAZB  FOURCC_NATIVE('Q','A','Z','B')
 #define T_QTEL	FOURCC_NATIVE('Q','T','E','L')
 #define T_RASP  FOURCC_NATIVE('R','A','S','P')
 #define T_RAZA	FOURCC_NATIVE('R','A','Z','A')
