@@ -10,7 +10,9 @@
 #define dirtypag sdbm__dirtypag
 #define flush_dirtypag sdbm__flush_dirtypag
 #define setcache sdbm__setcache
+#define getcache sdbm__getcache
 #define setwdelay sdbm__setwdelay
+#define getwdelay sdbm__getwdelay
 #define cachepag sdbm__cachepag
 
 void lru_init(DBM *);
@@ -20,7 +22,9 @@ bool dirtypag(DBM *, bool);
 bool flushpag(DBM *, char *, long);
 ssize_t flush_dirtypag(DBM *);
 int setcache(DBM *, long);
+long getcache(const DBM *);
 int setwdelay(DBM *, bool);
+bool getwdelay(const DBM *);
 bool cachepag(DBM *, char *, long);
 char *lru_cached_page(DBM *, long);
 void lru_discard(DBM *, long);
