@@ -1,13 +1,13 @@
 #include "common.h"
 
+#include "lib/progname.h"
+
 G_GNUC_PRINTF(1, 2) void
 oops(char *fmt, ...)
 {
 	va_list args;
-	extern char *progname;
 
-	if (progname)
-		fprintf(stderr, "%s: ", progname);
+	fprintf(stderr, "%s: ", getprogname());
 
 	va_start(args, fmt);
 	vfprintf(stderr, fmt, args);
