@@ -47,9 +47,11 @@
 #include "lib/log.h"
 #include "lib/misc.h"
 #include "lib/omalloc.h"
+#include "lib/palloc.h"
 #include "lib/parse.h"
 #include "lib/str.h"
 #include "lib/stringify.h"
+#include "lib/tmalloc.h"
 #include "lib/vmm.h"
 #include "lib/xmalloc.h"
 #include "lib/zalloc.h"
@@ -95,7 +97,7 @@ read_memory(int fd[2], const unsigned char *addr, size_t length,
 	}
 }
 
-static inline enum shell_reply	/* "inline" to avoid warning if unused */
+static enum shell_reply	G_GNUC_UNUSED	/* May be unused */
 shell_exec_memory_dump(struct gnutella_shell *sh,
 	int argc, const char *argv[])
 {
@@ -139,7 +141,7 @@ shell_exec_memory_dump(struct gnutella_shell *sh,
 		read_memory(fd, addr, length, data, sizeof data, valid);
 
 		str_cpy(s, pointer_to_string(addr));
-		str_cat(s, "  ");
+		STR_CAT(s, "  ");
 
 		for (i = 0; i < G_N_ELEMENTS(data); i++) {
 			if (length > i) {
@@ -150,13 +152,13 @@ shell_exec_memory_dump(struct gnutella_shell *sh,
 					str_putc(s, hex_digit(c & 0x0f));
 					str_putc(s, ' ');
 				} else {
-					str_cat(s, "XX ");
+					STR_CAT(s, "XX ");
 				}
 			} else {
-				str_cat(s, "   ");
+				STR_CAT(s, "   ");
 			}
 		}
-		str_cat(s, " |");
+		STR_CAT(s, " |");
 
 		for (i = 0; i < G_N_ELEMENTS(data); i++) {
 			if (length > i) {
@@ -167,7 +169,7 @@ shell_exec_memory_dump(struct gnutella_shell *sh,
 				str_putc(s, ' ');
 			}
 		}
-		str_cat(s, "|\n");
+		STR_CAT(s, "|\n");
 		shell_write(sh, str_2c(s));
 
 		if (length < G_N_ELEMENTS(data))
@@ -300,6 +302,39 @@ shell_exec_memory_show_options(struct gnutella_shell *sh,
 	return memory_run_showerv(sh, v, G_N_ELEMENTS(v));
 }
 
+static enum shell_reply
+shell_exec_memory_show_hole(struct gnutella_shell *sh,
+	int argc, const char *argv[])
+{
+	shell_check(sh);
+	g_assert(argv);
+	g_assert(argc > 0);
+
+	return memory_run_shower(sh, vmm_dump_hole_log, "VMM ");
+}
+
+static enum shell_reply
+shell_exec_memory_show_magazines(struct gnutella_shell *sh,
+	int argc, const char *argv[])
+{
+	shell_check(sh);
+	g_assert(argv);
+	g_assert(argc > 0);
+
+	return memory_run_shower(sh, tmalloc_dump_magazines_log, "TMALLOC ");
+}
+
+static enum shell_reply
+shell_exec_memory_show_pcache(struct gnutella_shell *sh,
+	int argc, const char *argv[])
+{
+	shell_check(sh);
+	g_assert(argv);
+	g_assert(argc > 0);
+
+	return memory_run_shower(sh, vmm_dump_pcache_log, "VMM ");
+}
+
 static enum shell_reply
 shell_exec_memory_show_pmap(struct gnutella_shell *sh,
 	int argc, const char *argv[])
@@ -311,6 +346,17 @@ shell_exec_memory_show_pmap(struct gnutella_shell *sh,
 	return memory_run_shower(sh, vmm_dump_pmap_log, "VMM ");
 }
 
+static enum shell_reply
+shell_exec_memory_show_pools(struct gnutella_shell *sh,
+	int argc, const char *argv[])
+{
+	shell_check(sh);
+	g_assert(argv);
+	g_assert(argc > 0);
+
+	return memory_run_shower(sh, palloc_dump_pool_log, "PALLOC ");
+}
+
 static enum shell_reply
 shell_exec_memory_show_xmalloc(struct gnutella_shell *sh,
 	int argc, const char *argv[])
@@ -349,8 +395,12 @@ shell_exec_memory_show(struct gnutella_shell *sh,
 		return shell_exec_memory_show_## name(sh, argc - 1, argv + 1); \
 } G_STMT_END
 
+	CMD(hole);
+	CMD(magazines);
 	CMD(options);
+	CMD(pcache);
 	CMD(pmap);
+	CMD(pools);
 	CMD(xmalloc);
 	CMD(zones);
 
@@ -383,6 +433,16 @@ shell_exec_memory_stats_halloc(struct gnutella_shell *sh,
 	return memory_run_opt_shower(sh, halloc_dump_stats_log, "HALLOC ", opt);
 }
 
+static enum shell_reply
+shell_exec_memory_stats_palloc(struct gnutella_shell *sh,
+	unsigned opt, unsigned which)
+{
+	if (which & STATS_USAGE)
+		return memory_stats_unsupported(sh, "palloc", STATS_USAGE_STR);
+
+	return memory_run_opt_shower(sh, palloc_dump_stats_log, "PALLOC ", opt);
+}
+
 static enum shell_reply
 shell_exec_memory_stats_vmm(struct gnutella_shell *sh,
 	unsigned opt, unsigned which)
@@ -393,6 +453,16 @@ shell_exec_memory_stats_vmm(struct gnutella_shell *sh,
 	return memory_run_opt_shower(sh, vmm_dump_stats_log, "VMM ", opt);
 }
 
+static enum shell_reply
+shell_exec_memory_stats_tmalloc(struct gnutella_shell *sh,
+	unsigned opt, unsigned which)
+{
+	if (which & STATS_USAGE)
+		return memory_stats_unsupported(sh, "tmalloc", STATS_USAGE_STR);
+
+	return memory_run_opt_shower(sh, tmalloc_dump_stats_log, "TMALLOC ", opt);
+}
+
 static enum shell_reply
 shell_exec_memory_stats_xmalloc(struct gnutella_shell *sh,
 	unsigned opt, unsigned which)
@@ -459,6 +529,8 @@ shell_exec_memory_stats(struct gnutella_shell *sh,
 } G_STMT_END
 
 	CMD(halloc);
+	CMD(palloc);
+	CMD(tmalloc);
 	CMD(vmm);
 	CMD(xmalloc);
 	CMD(zalloc);
@@ -492,7 +564,7 @@ shell_exec_memory_check_xmalloc(struct gnutella_shell *sh,
 		shell_write(sh, ".\n");
 
 	shell_write_linef(sh, REPLY_READY, "Found %zu freelist%s in error",
-		errors, 1 == errors ? "" : "s");
+		errors, plural(errors));
 
 	return REPLY_READY;
 }
@@ -674,12 +746,17 @@ shell_help_memory(int argc, const char *argv[])
 		}
 		else if (0 == ascii_strcasecmp(argv[1], "show")) {
 			return
+				"memory show hole      # display VMM first known hole\n"
+				"memory show magazines # display thread magazine information\n"
 				"memory show options   # display memory options\n"
+				"memory show pcache    # display VMM page cache\n"
 				"memory show pmap      # display VMM pmap\n"
+				"memory show pools     # display allocation pools\n"
 				"memory show xmalloc   # display xmalloc() freelist info\n"
 				"memory show zones     # display zone usage\n";
 		} else if (0 == ascii_strcasecmp(argv[1], "stats")) {
-			return "memory stats [-pu] halloc|omalloc|vmm|xmalloc|zalloc\n"
+			return "memory stats [-pu] "
+				"halloc|omalloc|palloc|tmalloc|vmm|xmalloc|zalloc\n"
 				"show statistics about specified memory sub-system\n"
 				"-p : pretty-print numbers with thousands separators\n"
 				"-u : show allocation usage statistics, if available\n";
@@ -693,8 +770,8 @@ shell_help_memory(int argc, const char *argv[])
 		"memory dump ADDRESS LENGTH\n"
 #endif
 		"memory check xmalloc\n"
-		"memory show options|pmap|xmalloc|zones\n"
-		"memory stats [-pu] omalloc|vmm|xmalloc|zalloc\n"
+		"memory show hole|magazines|options|pmap|pools|xmalloc|zones\n"
+		"memory stats [-pu] omalloc|palloc|tmalloc|vmm|xmalloc|zalloc\n"
 		"memory usage zone <size> on|off|show\n"
 		;
 	}
