@@ -2203,10 +2203,11 @@ filter_apply(filter_t *filter, struct filter_context *ctx, filter_result_t *res)
 					)
                         match = TRUE;
                     if (i == REG_ESPACE)
-                        g_warning("regexp memory overflow");
+                        g_warning("%s(): regexp memory overflow", G_STRFUNC);
                     break;
                 default:
-                    g_error("Unknown text rule type: %d", r->u.text.type);
+                    g_error("%s(): unknown text rule type: %d",
+						G_STRFUNC, r->u.text.type);
                 }
                 break;
 			}
@@ -2547,10 +2548,10 @@ filter_init_presets(void)
 		filesize_t minsize;
 	} tab[] = {
 		{ N_("<Archive>"),	  "[.](bz2|gz|zip|rar|iso|7z)$", 0 },
-		{ N_("<Audio>"), 	  "[.](mp3|m4a|ogg|oga|flac)$", 1000000 },
+		{ N_("<Audio>"), 	  "[.](mp3|m4a|ogg|oga|opus|flac)$", 1000000 },
 		{ N_("<Image>"), 	  "[.](bmp|gif|jpg|jpeg|png|psd|tif|tiff)$", 0 },
 		{ N_("<Literature>"), "[.](pdf|doc|lit|djvu|ps|txt)$", 10000 },
-		{ N_("<Video>"), 	  "[.](avi|mpg|mp4|mpeg|mkv|ogm|ogv)$", 10000000 },
+		{ N_("<Video>"), 	  "[.](avi|mpg|mp4|mpeg|mkv|ogm|ogv|webm)$", 10000000 },
 	};
 	unsigned i;
 
