@@ -161,7 +161,7 @@ add_column(GtkTreeView *treeview, gint column_id, gint width, gfloat xalign,
 	g_object_set(renderer,
 		"xalign", xalign,
 		"ypad", GUI_CELL_RENDERER_YPAD,
-		(void *) 0);
+		NULL_PTR);
 	column = gtk_tree_view_column_new_with_attributes(label, renderer,
 		"text", column_id,
 		NULL);
@@ -171,7 +171,7 @@ add_column(GtkTreeView *treeview, gint column_id, gint width, gfloat xalign,
 		"reorderable", TRUE,
 		"resizable", TRUE,
 		"sizing", GTK_TREE_VIEW_COLUMN_FIXED,
-		(void *) 0);
+		NULL_PTR);
 	gtk_tree_view_append_column(treeview, column);
 }
 
