@@ -793,7 +793,7 @@ tree_view_restore_widths(GtkTreeView *treeview, property_t prop)
 		gui_prop_get_guint32(prop, &width, i, 1);
 		g_object_set(G_OBJECT(c),
 			"fixed-width", MAX(1, (gint32) width),
-			(void *) 0);
+			NULL_PTR);
 	}
 }
 
