@@ -1,6 +1,4 @@
 /*
- * $Id: settings.h,v 1.3 2004/11/21 21:12:42 wyldfire Exp $
- *
  * Copyright (c) 2001-2003, Richard Eckart
  *
  *----------------------------------------------------------------------
@@ -33,10 +31,14 @@
 #include "if/gui_property.h"
 
 void settings_gui_init(void);
-void settings_gui_init_late(void);
+void settings_gui_restore_panes(void);
 const gchar *settings_gui_config_dir(void);
 GtkTooltips *settings_gui_tooltips(void);
 void settings_gui_save_if_dirty(void);
 void settings_gui_shutdown(void);
+gboolean show_metric_units(void);
+
+void ancient_version_dialog_show(void);
+void ancient_version_dialog_hide(void);
 
 #endif /* _gtk_settings_h_ */
