@@ -38,6 +38,8 @@
 #include "xattr.h"
 #include "xnode.h"
 
+#include "if/gen/vxml.h"
+
 struct vxml_parser;
 typedef struct vxml_parser vxml_parser_t;
 
@@ -50,57 +52,6 @@ typedef struct vxml_parser vxml_parser_t;
 #define VXML_O_STRIP_BLANKS		(1 << 3)  /**< Strip leading/ending blanks */
 #define VXML_O_NO_DUP_ATTR		(1 << 4)  /**< Forbid duplicate attributes */
 
-/**
- * Error codes.
- */
-typedef enum {
-	VXML_E_OK = 0,						/**< No error */
-	VXML_E_UNSUPPORTED_BYTE_ORDER,		/**< Unsupported byte order */
-	VXML_E_UNSUPPORTED_CHARSET,			/**< Unsupported character set */
-	VXML_E_TRUNCATED_INPUT,				/**< Truncated input stream */
-	VXML_E_EXPECTED_NAME_START,			/**< Expected a valid name start */
-	VXML_E_INVALID_CHAR_REF,			/**< Invalid character reference */
-	VXML_E_INVALID_CHARACTER,			/**< Invalid Unicode character */
-	VXML_E_INVALID_NAME_CHARACTER,		/**< Invalid character in name */
-	VXML_E_UNKNOWN_ENTITY_REF,			/**< Unknown entity reference */
-	VXML_E_UNEXPECTED_CHARACTER,		/**< Unexpected character */
-	VXML_E_UNEXPECTED_WHITESPACE,		/**< Unexpected white space */
-	VXML_E_BAD_CHAR_IN_NAME,			/**< Bad character in name */
-	VXML_E_INVALID_TAG_NESTING,			/**< Invalid tag nesting */
-	VXML_E_EXPECTED_QUOTE,				/**< Was expecting a quote */
-	VXML_E_EXPECTED_GT,					/**< Was expecting a '>' */
-	VXML_E_EXPECTED_SPACE,				/**< Was expecting a space */
-	VXML_E_EXPECTED_LBRAK,				/**< Was expecting a '[' */
-	VXML_E_EXPECTED_RBRAK,				/**< Was expecting a ']' */
-	VXML_E_EXPECTED_TWO_MINUS,			/**< Was expecting '--' */
-	VXML_E_EXPECTED_DECL_TOKEN,			/**< Was expecting declaration token */
-	VXML_E_EXPECTED_NDATA_TOKEN,		/**< Was expecting 'NDATA' token */
-	VXML_E_EXPECTED_CDATA_TOKEN,		/**< Was expecting 'CDATA' token */
-	VXML_E_EXPECTED_COND_TOKEN,			/**< Expected 'INCLUDE' or 'IGNORE' */
-	VXML_E_EXPECTED_DOCTYPE_DECL,		/**< Was expecting DOCTYPE decl. */
-	VXML_E_UNEXPECTED_LT,				/**< Was not expecting a '<' */
-	VXML_E_UNEXPECTED_XML_PI,			/**< Spurious "<?xml ...> */
-	VXML_E_UNEXPECTED_TAG_END,			/**< Unexpected tag end */
-	VXML_E_NESTED_DOCTYPE_DECL,			/**< Was not expecting DOCTYPE decl. */
-	VXML_E_INVALID_VERSION,				/**< Invalid version number */
-	VXML_E_VERSION_OUT_OF_RANGE,		/**< Version number out of range */
-	VXML_E_UNKNOWN_CHAR_ENCODING_NAME,	/**< Unknown character encoding name */
-	VXML_E_INVALID_CHAR_ENCODING_NAME,	/**< Invalid character encoding name */
-	VXML_E_ILLEGAL_CHAR_BYTE_SEQUENCE,	/**< Illegal character byte sequence */
-	VXML_E_UNREADABLE_CHAR_ENCODING,	/**< Unreadable input */
-	VXML_E_USER,						/**< User-defined error */
-	VXML_E_DUP_ATTRIBUTE,				/**< Duplicate attribute */
-	VXML_E_DUP_DEFAULT_NAMESPACE,		/**< Duplicate default namespace */
-	VXML_E_BAD_CHAR_IN_NAMESPACE,		/**< Bad character in namespace */
-	VXML_E_NAMESPACE_REDEFINITION,		/**< Invalid namespace redefinition */
-	VXML_E_UNKNOWN_NAMESPACE,			/**< Unknown namespace prefix */
-	VXML_E_EMPTY_NAME,					/**< Empty name */
-	VXML_E_IO,							/**< I/O error */
-	VXML_E_ENTITY_RECURSION,			/**< Possible entity recursion */
-
-	VXML_E_MAX
-} vxml_error_t;
-
 /**
  * Start plain element callback signature.
  *
@@ -194,14 +145,6 @@ struct vxml_token {
 	unsigned id;			/**< Corresponding token */
 };
 
-/**
- * A parsing token description, the mapping between a name and a number.
- */
-struct vxml_parser_token {
-	const char *name;
-	unsigned value;
-};
-
 /*
  * Public constants.
  */
@@ -218,7 +161,6 @@ void vxml_test(void);
 void set_vxml_debug(uint32 level);
 bool vxml_debugging(uint32 level) G_GNUC_PURE;
 
-const char *vxml_strerror(vxml_error_t error);
 const char *vxml_parser_strerror(const vxml_parser_t *vp, vxml_error_t error);
 vxml_parser_t *vxml_parser_make(const char *name, uint32 options);
 void vxml_parser_free(vxml_parser_t *vp);
@@ -235,9 +177,6 @@ vxml_error_t vxml_parse_callbacks_tokens(vxml_parser_t *vp,
 	struct vxml_token *tvec, size_t tlen, void *data);
 vxml_error_t vxml_parse_tree(vxml_parser_t *vp, xnode_t **root);
 
-unsigned vxml_token_lookup(const char *name,
-	const struct vxml_parser_token *tokens, size_t len);
-
 void vxml_parser_error(vxml_parser_t *vp,
 		const char *errstr, ...) G_GNUC_PRINTF(2, 3);
 unsigned vxml_parser_depth(const vxml_parser_t *vp);
